/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import java.io.*;
import java.util.*; //pour StringTokenizer

public class AnatoliaResetsReader extends AnatoliaAreReader
{
  public AnatoliaResetsReader(AreaMap areaMap)
  {
    super(areaMap);
  }

  public void read(BufferedReader input) throws IOException
  {
     AnatoliaMobile lastMobile = null;
     AnatoliaObject lastObject = null;

     String line = super.trim(input);
     StringTokenizer tokens = null;
     LinkedList lateInstances = new LinkedList(); //pair<obj, container>

     while(line.startsWith("D") || line.startsWith("M") || line.startsWith("E") ||
           line.startsWith("G") || line.startsWith("O") || line.startsWith("P") ||
           line.startsWith("R")
          )
     {
       if (line.startsWith("D"))
       {
          tokens = new StringTokenizer(line);
          tokens.nextToken(); //reads "D"
          //<:number> <room-vnum:number> <door:number> <state:number>
          int number = MyInteger.parseInt(tokens.nextToken());
          int room_vnum = MyInteger.parseInt(tokens.nextToken());
          int door = MyInteger.parseInt(tokens.nextToken());
          int state = MyInteger.parseInt(tokens.nextToken());
          AnatoliaRoom room = (AnatoliaRoom) areaMap.theRooms.anatoliaCollection.getAnatoliaThingOfVnum(room_vnum);
          if (room.doors[door] == null)
          {
            //System.out.println("Bug");
          }
          else
          {
            room.doors[door].doorState = (byte) state;
            room.doors[door].doorType  = AnatoliaConsts.EXIT_WITH_DOOR;
          }
       }
       else if (line.startsWith("M"))
       {
         //M 0 <mobile vnum> <global mob limit> <room vnum> <local mob limit>
          tokens = new StringTokenizer(line);
          tokens.nextToken(); //reads "M"
          tokens.nextToken(); //reads "0"
          int mob_vnum = MyInteger.parseInt(tokens.nextToken());
          int global_limit = MyInteger.parseInt(tokens.nextToken());
          int room_vnum = MyInteger.parseInt(tokens.nextToken());
          int local_limit = MyInteger.parseInt(tokens.nextToken());
          AnatoliaMobile mobile = (AnatoliaMobile) areaMap.theMobiles.getAnatoliaThingOfVnum(mob_vnum);
          if (mobile == null) //extern mobile
          {
            mobile = new AnatoliaMobile(areaMap.theMobiles);
            mobile.setExtern(true);
            mobile.imposeVnum(mob_vnum);
          }
          AnatoliaRoom room = (AnatoliaRoom) areaMap.theRooms.anatoliaCollection.getAnatoliaThingOfVnum(room_vnum);

          InstanceOfMobile instance = new InstanceOfMobile(mobile);
          mobile.addInstance(instance, room);
          areaMap.objectTree.addNode(instance, room);
          lastMobile = mobile;
       }
       else if (line.startsWith("E"))
       {
          //E 0 obj_vnum 0 location
          tokens = new StringTokenizer(line);
          tokens.nextToken(); //reads "E"
          tokens.nextToken(); //reads "0"
          int obj_vnum = MyInteger.parseInt(tokens.nextToken());
          tokens.nextToken(); //reads "0"
          int location = Math.max(0,AreaBuilderConsts.findInTab(AnatoliaConsts.getWearCodes(),tokens.nextToken()));
          location = Math.min(location, AnatoliaConsts.wearTexts.length);
          AnatoliaObject object = (AnatoliaObject) areaMap.theObjects.getAnatoliaThingOfVnum(obj_vnum);
          if (object == null) //extern object
          {
            object = new AnatoliaObject(areaMap.theObjects);
            object.setExtern(true);
            object.imposeVnum(obj_vnum);
          }
          InstanceOfObject instance = new InstanceOfObject(object, location);
          object.addInstance(instance);
          areaMap.objectTree.addNode(instance, (InstanceOfMobile) lastMobile.instances.getLast());
       }
       else if (line.startsWith("G"))
       {
          //G 0 obj_vnum 0
          tokens = new StringTokenizer(line);
          tokens.nextToken(); //reads "G"
          tokens.nextToken(); //reads "0"
          int obj_vnum = MyInteger.parseInt(tokens.nextToken());
          tokens.nextToken(); //reads "0"
          AnatoliaObject object = (AnatoliaObject) areaMap.theObjects.getAnatoliaThingOfVnum(obj_vnum);
          if (object == null) //extern object
          {
            object = new AnatoliaObject(areaMap.theObjects);
            object.setExtern(true);
            object.imposeVnum(obj_vnum);
          }
          InstanceOfObject instance = new InstanceOfObject(object, -1);
          object.addInstance(instance);
          areaMap.objectTree.addNode(instance, (InstanceOfMobile) lastMobile.instances.getLast());
       }
       else if (line.startsWith("O"))
       {
          //O 0 <object vnum> 0 <room vnum>
          tokens = new StringTokenizer(line);
          tokens.nextToken(); //reads "O"
          tokens.nextToken(); //reads "0"
          int obj_vnum = MyInteger.parseInt(tokens.nextToken());
          tokens.nextToken(); //reads "0"
          int room_vnum = MyInteger.parseInt(tokens.nextToken());
          AnatoliaObject object = (AnatoliaObject) areaMap.theObjects.getAnatoliaThingOfVnum(obj_vnum);
          if (object == null) //extern object
          {
            object = new AnatoliaObject(areaMap.theObjects);
            object.setExtern(true);
            object.imposeVnum(obj_vnum);
          }
          AnatoliaRoom room = (AnatoliaRoom) areaMap.theRooms.anatoliaCollection.getAnatoliaThingOfVnum(room_vnum);
          InstanceOfObject instance = new InstanceOfObject(object, -1);
          object.addInstance(instance);
          areaMap.objectTree.addNode(instance, room);
          lastObject = object;
       }
       else if (line.startsWith("P"))
       {
         //P 0 obj 0 container nb_times
          tokens = new StringTokenizer(line);
          tokens.nextToken(); //reads "P"
          tokens.nextToken(); //reads "0"
          int obj_vnum = MyInteger.parseInt(tokens.nextToken());
          tokens.nextToken(); //reads "0"
          int container_vnum = MyInteger.parseInt(tokens.nextToken());
          int nbtimes = 1;
          if (tokens.hasMoreTokens())
          {
            nbtimes = MyInteger.parseInt(tokens.nextToken());
            if (nbtimes<=0) nbtimes = 1;
          }
          AnatoliaObject object = (AnatoliaObject) areaMap.theObjects.getAnatoliaThingOfVnum(obj_vnum);
          if (object == null) //extern object
          {
            object = new AnatoliaObject(areaMap.theObjects);
            object.setExtern(true);
            object.imposeVnum(obj_vnum);
          }
          AnatoliaObject container = (AnatoliaObject) areaMap.theObjects.getAnatoliaThingOfVnum(container_vnum);
          if (container == null) //extern object
          {
            container = new AnatoliaObject(areaMap.theObjects);
            container.setExtern(true);
            container.imposeVnum(container_vnum);
          }
          for(int i=0 ; i<nbtimes ; ++i)
          {
            InstanceOfObject instance = new InstanceOfObject(object, -1);
            object.addInstance(instance);
            lateInstances.add(new Pair(instance, container));
          }
       }
       else if (line.startsWith("R"))
       {
          //0 <room-vnum:number> <random up to...>
          tokens = new StringTokenizer(line);
          tokens.nextToken(); //reads "R"
          tokens.nextToken(); //reads "0"
          int room_vnum = MyInteger.parseInt(tokens.nextToken());
          int random = MyInteger.parseInt(tokens.nextToken());
          AnatoliaRoom room = (AnatoliaRoom) areaMap.theRooms.anatoliaCollection.getAnatoliaThingOfVnum(room_vnum);
          room.flags[AnatoliaConsts.ROOM_FLAG_RANDOM] = true;
          room.randomUpTo = (byte) Math.min(5,random);
       }
       line = super.trim(input);
     }
     //late instances
     Iterator it = lateInstances.iterator();
     Pair pair = null;
     while(it.hasNext())
     {
       pair = (Pair) it.next();
       AnatoliaObject container = (AnatoliaObject) pair.getSecond();
       Iterator it2 = container.instances.iterator();
       while(it2.hasNext())
       {
         areaMap.objectTree.addNode((InstanceOfObject) pair.getFirst(), (InstanceOfObject) it2.next());
       }
     }

     //expects the end ("S")
     while(!line.startsWith("S")) line = super.trim(input);
  }
}