/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import java.awt.*;
import java.awt.color.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*; //JTable
import javax.swing.border.TitledBorder; //border of Panel
import java.util.*; //LinkedList

public class AnatoliaRoomDialog extends JDialog implements ActionListener
{
   AreaMap areaMap = null;
   private SelectionOfRooms selectionOfRooms = null; //the properties will apply to these
   private AnatoliaRoom anatoliaRoom = null; //or to this
   private LinkedList unknownProperties = new LinkedList(); //when some rooms of the selection do not
                                                            //have the same value for a flag, we log
                                                            //it

   private JTabbedPane tabbedPane = new JTabbedPane();
   private RoomNumbering roomNumbering = new RoomNumbering();
   private RoomDescription roomDescription = new RoomDescription();
   private Flags flagsPanel = new Flags();
   private SectorTypes sectorPanel = new SectorTypes();
   private HPMPRecovery HPMPRecoveryPanel = new HPMPRecovery();
   private Exits exits = new Exits();
   private JButton okButton = new JButton("OK");
   private JButton cancelButton = new JButton("Cancel");

   public AnatoliaRoomDialog(Dialog owner, String title, boolean modal, AreaMap areaMap)
   {
      super(owner, title, modal);
      this.areaMap = areaMap;
      Container contentPane = getContentPane();
      contentPane.setLayout(new GridBagLayout());
      GridBagConstraints constraint = new GridBagConstraints();

      constraint.gridx=0;constraint.gridy=0;
      constraint.gridwidth=2;constraint.gridheight=1;
      constraint.weightx=100;constraint.weighty=90;
      contentPane.add(tabbedPane, constraint);

        tabbedPane.addTab("Room Numbering", roomNumbering);
        tabbedPane.addTab("Room Description", roomDescription);

        JPanel flagsAndSectorTypePane = new JPanel();
        Box box = Box.createVerticalBox();
        box.add(flagsPanel);
        box.add(sectorPanel);
        box.add(HPMPRecoveryPanel);
        flagsAndSectorTypePane.add(box);
        tabbedPane.addTab("Room flags", flagsAndSectorTypePane);

        tabbedPane.addTab("Doors", exits);

      constraint.gridx=0;constraint.gridy=1;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=50;constraint.weighty=10;
      contentPane.add(okButton, constraint);

      constraint.gridx=1;constraint.gridy=1;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=50;constraint.weighty=10;
      contentPane.add(cancelButton, constraint);

      roomNumbering.outsideRoom.addActionListener(this);
      roomNumbering.autoVnum.addActionListener(this);
      okButton.addActionListener(this);
      cancelButton.addActionListener(this);
      this.pack();
   }

   public void actionPerformed(ActionEvent e)
   {
      if (e.getSource() == roomNumbering.outsideRoom)
      {
         boolean isOutsideRoom = ((JCheckBox) e.getSource()).isSelected();
         this.tabbedPane.setEnabledAt(0,true);
         this.tabbedPane.setEnabledAt(1,true);
         this.tabbedPane.setEnabledAt(2,!isOutsideRoom);
         this.tabbedPane.setEnabledAt(3,!isOutsideRoom);
         roomNumbering.autoVnum.setEnabled(!isOutsideRoom);
         roomNumbering.vnum.setEnabled(isOutsideRoom || !roomNumbering.autoVnum.isSelected());
      }
      else if (e.getSource() == roomNumbering.autoVnum)
      {
         roomNumbering.vnum.setEnabled(roomNumbering.outsideRoom.isSelected() || !roomNumbering.autoVnum.isSelected());
         if (roomNumbering.autoVnum.isSelected())
         {
            roomNumbering.vnum.setText(Integer.toString(anatoliaRoom.getVnum()));
         }
      }
      else if (e.getSource() == cancelButton)
      {
         setVisible(false);
      }
      else if (e.getSource() == okButton)
      {
         if (anatoliaRoom != null)
            setVisible(!applyProperties(anatoliaRoom));
         else if (selectionOfRooms != null)
            setVisible(!applyProperties(selectionOfRooms));
      }
   }

   public void showFor(SelectionOfRooms selectionOfRooms)
   {
      anatoliaRoom = null;
      this.selectionOfRooms = selectionOfRooms;
      if ((selectionOfRooms == null) || (selectionOfRooms.isEmpty())) return;

      this.tabbedPane.setEnabledAt(0,false);
      this.tabbedPane.setEnabledAt(1,false);
      this.tabbedPane.setEnabledAt(2,true);
      this.tabbedPane.setEnabledAt(3,false);
      this.tabbedPane.setSelectedIndex(2);

      if (selectionOfRooms.getSize() == 1)
      {
         Room room = (Room) selectionOfRooms.getIterator().next();
         showFor(room);
         return;
      }

      boolean[] sharedFlags = new boolean[this.flagsPanel.flags.length];
      for(int flag=0 ; flag<this.flagsPanel.sameFlags.length; ++flag)
         this.flagsPanel.sameFlags[flag] = true;

      Iterator it = selectionOfRooms.getIterator();
      Room room = (Room) it.next();
      for(int flag=0 ; flag<this.flagsPanel.sameFlags.length; ++flag)
         sharedFlags[flag] = room.anatoliaRoom.flags[flag];
      this.flagsPanel.sameRandomUpTo = true;
      byte sharedRandomUpTo = room.anatoliaRoom.randomUpTo;
      this.flagsPanel.randomUpToBox.setSelectedIndex(sharedRandomUpTo);
      this.flagsPanel.randomUpToBox.setEnabled(sharedFlags[AnatoliaConsts.ROOM_FLAG_RANDOM]);
      this.sectorPanel.sameSectorType = true;
      byte sharedSectorType = room.anatoliaRoom.sectorType;
      int sharedHPRecovery =  room.anatoliaRoom.HPRecovery;
      int sharedMPRecovery =  room.anatoliaRoom.MPRecovery;
      while (it.hasNext())
      {
         room = (Room) it.next();
         this.sectorPanel.sameSectorType =     this.sectorPanel.sameSectorType
                                           && (room.anatoliaRoom.sectorType == sharedSectorType);
         this.flagsPanel.sameRandomUpTo =     this.flagsPanel.sameRandomUpTo
                                           && (room.anatoliaRoom.randomUpTo == sharedRandomUpTo);
         for(int flag=0 ; flag<this.flagsPanel.sameFlags.length ; ++flag)
           this.flagsPanel.sameFlags[flag] =    this.flagsPanel.sameFlags[flag]
                                            && (room.anatoliaRoom.flags[flag] == sharedFlags[flag]);
         this.HPMPRecoveryPanel.sameHPRecovery = this.HPMPRecoveryPanel.sameHPRecovery && (room.anatoliaRoom.HPRecovery == sharedHPRecovery);
         this.HPMPRecoveryPanel.sameMPRecovery = this.HPMPRecoveryPanel.sameMPRecovery && (room.anatoliaRoom.MPRecovery == sharedMPRecovery);
      }

      for(int i=0; i < this.flagsPanel.sameFlags.length ; ++i)
      {
         if (!this.flagsPanel.sameFlags[i]) this.flagsPanel.flags[i].setForeground(Color.red);
      }
      if (!this.flagsPanel.sameRandomUpTo)
         this.flagsPanel.randomUpToBox.setForeground(Color.red);

      this.HPMPRecoveryPanel.hpSlider.setValue(sharedHPRecovery);
      if (!this.HPMPRecoveryPanel.sameHPRecovery)
         this.HPMPRecoveryPanel.hpLabel.setForeground(Color.red);
      else
         this.HPMPRecoveryPanel.hpLabel.setForeground(Color.black);

      this.HPMPRecoveryPanel.mpSlider.setValue(sharedMPRecovery);
      if (!this.HPMPRecoveryPanel.sameMPRecovery)
         this.HPMPRecoveryPanel.mpLabel.setForeground(Color.red);
      else
         this.HPMPRecoveryPanel.mpLabel.setForeground(Color.black);

      if (this.sectorPanel.sameSectorType)
         this.sectorPanel.sectors[sharedSectorType].setSelected(true);
      else
      {
         JRadioButton tempButton = null;
         for(int j=0; j<this.sectorPanel.sectors.length ; ++j)
         {
            tempButton = this.sectorPanel.sectors[j];
            this.sectorPanel.sectorGroup.remove(tempButton);
            tempButton.setSelected(false);
            tempButton.setForeground(Color.red);
            this.sectorPanel.sectorGroup.add(tempButton);
         }
      }

      pack();
      Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
      Dimension mySize = this.getSize();
      this.setLocation((int) (screenSize.getWidth()-mySize.getWidth())/2,
                       (int) (screenSize.getHeight()-mySize.getHeight())/2
                      );
      setVisible(true);
   }

   public void showFor(Room room, StringLocation stringLocation)
   {
     showFor(room);
   }

   public void showFor(Room room)
   {
      selectionOfRooms = null;
      this.anatoliaRoom = room.anatoliaRoom;
      if (room == null) return;

      if (anatoliaRoom.isExtern())
      {
        this.tabbedPane.setSelectedIndex(0);
        this.tabbedPane.setEnabledAt(0,true);
        this.tabbedPane.setEnabledAt(1,true);
        this.tabbedPane.setEnabledAt(2,false);
        this.tabbedPane.setEnabledAt(3,false);
      }
      else
      {
        this.tabbedPane.setSelectedIndex(1);
        this.tabbedPane.setEnabledAt(0,true);
        this.tabbedPane.setEnabledAt(1,true);
        this.tabbedPane.setEnabledAt(2,true);
        this.tabbedPane.setEnabledAt(3,true);
      }

      this.roomNumbering.vnum.setText(Integer.toString(room.anatoliaRoom.getVnum()));
      this.roomNumbering.outsideRoom.setSelected(anatoliaRoom.isExtern());
      this.roomNumbering.autoVnum.setSelected(room.autoVnum);
      this.roomNumbering.vnum.setEnabled(room.anatoliaRoom.isExtern() || !room.autoVnum);

      this.roomDescription.roomName.setText(room.anatoliaRoom.roomName);
      this.roomDescription.roomDesc.setText(room.anatoliaRoom.roomDescription);

      for (int i=0 ; i<this.roomDescription.extraDescriptionPanel.table.getRowCount() ; ++i)
      {
         this.roomDescription.extraDescriptionPanel.table.setValueAt("",i,0);
         this.roomDescription.extraDescriptionPanel.table.setValueAt("",i,1);
      }
      Iterator it = room.anatoliaRoom.extras.listIterator();
      int j=0;
      Pair pair = null;
      while(it.hasNext())
      {
         pair = (Pair) it.next();
         this.roomDescription.extraDescriptionPanel.table.setValueAt(pair.getFirst(),j,0);
         this.roomDescription.extraDescriptionPanel.table.setValueAt(pair.getSecond(),j,1);
         ++j;
      }

      for(int i=0 ; i<this.flagsPanel.flags.length ; ++i)
      {
         this.flagsPanel.flags[i].setForeground(Color.black);
         this.flagsPanel.flags[i].setSelected(room.anatoliaRoom.flags[i]);
      }

      this.flagsPanel.randomUpToBox.setForeground(Color.black);
      this.flagsPanel.randomUpToBox.setSelectedIndex(room.anatoliaRoom.randomUpTo);
      this.flagsPanel.randomUpToBox.setEnabled(room.anatoliaRoom.flags[AnatoliaConsts.ROOM_FLAG_RANDOM]);
      for(int i=0 ; i<this.sectorPanel.sectors.length ; ++i)
         this.sectorPanel.sectors[i].setForeground(Color.black);
      this.sectorPanel.sectors[room.anatoliaRoom.sectorType].setSelected(true);

      this.HPMPRecoveryPanel.hpLabel.setForeground(Color.black);
      this.HPMPRecoveryPanel.mpLabel.setForeground(Color.black);
      this.HPMPRecoveryPanel.hpSlider.setValue(Math.max(1,room.anatoliaRoom.HPRecovery));
      this.HPMPRecoveryPanel.mpSlider.setValue(Math.max(1,room.anatoliaRoom.MPRecovery));

      for(int i=0 ; i<6 ; ++i)
      {
         if (room.anatoliaRoom.doors[i] == null)
         {
            this.exits.tabbedPane.getComponentAt(i).setEnabled(false);
            this.exits.tabbedPane.setEnabledAt(i, false);
         }
         else
         {
            this.exits.tabbedPane.getComponentAt(i).setEnabled(true);
            this.exits.tabbedPane.setEnabledAt(i, true);
            this.exits.theExits[i].doorType.setSelected(room.anatoliaRoom.doors[i].doorType == AnatoliaConsts.EXIT_WITH_DOOR);
            this.exits.theExits[i].autoSetSymetricDoor.setSelected(room.anatoliaRoom.doors[i].autoSetSymetricDoor);
            this.exits.theExits[i].setDoorEnabled(room.anatoliaRoom.doors[i].doorType == AnatoliaConsts.EXIT_WITH_DOOR);
            this.exits.theExits[i].doorState.setSelectedIndex(room.anatoliaRoom.doors[i].doorState);
            this.exits.theExits[i].doorSize.setSelectedIndex(room.anatoliaRoom.doors[i].doorSize);
            this.exits.theExits[i].keyBox.removeAllItems();
            this.exits.theExits[i].keyBox.addItem("None");
            AnatoliaObject[] keys = areaMap.theObjects.getAllKeyObjects();
            for(int k=0 ; k<keys.length ; ++k)
            {
               this.exits.theExits[i].keyBox.addItem(keys[k]);
            }
            this.exits.theExits[i].keyBox.setEditable(false);
            AnatoliaObject key = (AnatoliaObject) areaMap.theObjects.getAnatoliaThingOfVnum(room.anatoliaRoom.doors[i].keyVnum);
            if (key == null)
            {
               room.anatoliaRoom.doors[i].keyVnum = 0;
               this.exits.theExits[i].keyBox.setSelectedIndex(0);
            }
            else
            {
               if (key.type != AnatoliaConsts.OBJECT_TYPE_KEY)
               {
                 room.anatoliaRoom.doors[i].keyVnum = 0;
                 this.exits.theExits[i].keyBox.setSelectedIndex(0);
               }
               else
               {
                  this.exits.theExits[i].keyBox.setSelectedItem(key);
               }
            }
            this.exits.theExits[i].description.setText(room.anatoliaRoom.doors[i].extraDescription);
            this.exits.theExits[i].doorKeyword.setText(room.anatoliaRoom.doors[i].doorKeyword);
            for(int k=0 ; k<this.exits.theExits[i].doorResets.length ; ++k)
               this.exits.theExits[i].doorResets[k].setSelected(room.anatoliaRoom.doors[i].doorResetFlags[k]);
         }
      }
      //look for first enabled
      for(int i=0 ; i<6 ; ++i)
      {
         if (this.exits.tabbedPane.isEnabledAt(i))
         {
            this.exits.tabbedPane.setSelectedIndex(i);
            break;
         }
      }

      pack();
      Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
      Dimension mySize = this.getSize();
      this.setLocation((int) (screenSize.getWidth()-mySize.getWidth())/2,
                       (int) (screenSize.getHeight()-mySize.getHeight())/2
                      );
      setVisible(true);
   }

   private boolean applyProperties(SelectionOfRooms selectedRooms)
   {
      if (this.flagsPanel.sameRandomUpTo)
      {
         Iterator it = selectedRooms.getIterator();
         while (it.hasNext())
         {
            ((Room) it.next()).anatoliaRoom.randomUpTo = (byte) this.flagsPanel.randomUpToBox.getSelectedIndex();
         }
      }

      int selectedSector = -1;
      for (int i=0; i<this.sectorPanel.sectors.length ; ++i)
      {
         if (this.sectorPanel.sectors[i].isSelected())
         {
            selectedSector = i;
            break;
         }
      }

      if (selectedSector != -1)
      {
         Iterator it = selectedRooms.getIterator();
         while (it.hasNext())
         {
            ((Room) it.next()).anatoliaRoom.sectorType = (byte) selectedSector;
         }
      }

      //!same HPRecovery : do nothing
      if (this.HPMPRecoveryPanel.sameHPRecovery)
      {
      }
      else //sameHPRecovery
      {
         Iterator it = selectionOfRooms.getIterator();
         while (it.hasNext()) ((Room) it.next()).anatoliaRoom.HPRecovery = Math.max(1,this.HPMPRecoveryPanel.hpSlider.getValue());
      }

      //!same MPRecovery : do nothing
      if (this.HPMPRecoveryPanel.sameMPRecovery)
      {
      }
      else //sameMPRecovery
      {
         Iterator it = selectionOfRooms.getIterator();
         while (it.hasNext()) ((Room) it.next()).anatoliaRoom.MPRecovery = Math.max(1,this.HPMPRecoveryPanel.mpSlider.getValue());
      }

      for(int i=0 ; i<this.flagsPanel.flags.length ; ++i)
      {
         if (this.flagsPanel.sameFlags[i])
         {
            Iterator it = selectionOfRooms.getIterator();
            while (it.hasNext())
            {
               ((Room) it.next()).anatoliaRoom.flags[i] = this.flagsPanel.flags[i].isSelected();
            }
         }

      }
      this.areaMap.objectTreeDialog.repaint();
      return true;
   }

   private boolean applyProperties(AnatoliaRoom anatoliaRoom)
   {
      if (anatoliaRoom == null) return true;

      int minVnumWithinArea = anatoliaRoom.myRoom.areaMap.areaNumber*1000;
      int tempInt = MyInteger.parseInt(roomNumbering.vnum.getText());

      if (roomNumbering.outsideRoom.isSelected())
      {
         if ((tempInt <0) || (tempInt > 99999))
         {
           JOptionPane.showMessageDialog(this, "Incorrect vnum (must be within [0;99999])",
                                               "Invalid properties",
                                               JOptionPane.ERROR_MESSAGE);
           return false;
         }
      }
      else // if (!roomNumbering.outsideRoom.isSelected())
      if ((tempInt <=minVnumWithinArea) || (tempInt >= minVnumWithinArea+1000))
      {
        JOptionPane.showMessageDialog(this, "Incorrect vnum (must be within ["+
                                            Integer.toString(minVnumWithinArea+1)+
                                            ";"+
                                            Integer.toString(minVnumWithinArea+999)+
                                            "])",
                                            "Invalid properties",
                                            JOptionPane.ERROR_MESSAGE);
        return false;
      }
      if (!anatoliaRoom.myCollection.setVnum(anatoliaRoom, tempInt))
      {
        AnatoliaRoom other = (AnatoliaRoom) anatoliaRoom.myCollection.getAnatoliaThingOfVnum(tempInt);
        if (other.myRoom.autoVnum)
        {
           int answer = JOptionPane.showConfirmDialog(this, "The vnum <"+Integer.toString(tempInt)+"> is already"+"\n"+
                                                      "used. Do you want to automatically change"+"\n"+
                                                      "the vnum of the OTHER room using it?"+"\n",
                                                      "Vnum conflict",
                                                      JOptionPane.YES_NO_CANCEL_OPTION);
           if ((answer == JOptionPane.CANCEL_OPTION) || (answer == JOptionPane.NO_OPTION))
              return false;
           else
           {
              other.myCollection.setToFirstFreeVnum(other);
              anatoliaRoom.myCollection.setVnum(anatoliaRoom, tempInt);
           }
        }
        else //other room has already this vnum IMPOSED!!!
        {
           JOptionPane.showMessageDialog(this, "This vnum is already reserved\n"+"for another room",
                                         "Vnum conflict", JOptionPane.ERROR_MESSAGE);
           return false;
        }
      }

      anatoliaRoom.setExtern(roomNumbering.outsideRoom.isSelected());
      anatoliaRoom.myRoom.autoVnum = roomNumbering.autoVnum.isSelected();
      anatoliaRoom.roomName = new String(roomDescription.roomName.getText());
      anatoliaRoom.roomDescription = new String(roomDescription.roomDesc.getText());

      anatoliaRoom.extras.clear();
      for(int i=0 ; i<this.roomDescription.extraDescriptionPanel.table.getRowCount() ; ++i)
      {
         if ( (i==this.roomDescription.extraDescriptionPanel.table.getEditingRow())
              || (this.roomDescription.extraDescriptionPanel.table.getValueAt(i,0) == null)
              || (this.roomDescription.extraDescriptionPanel.table.getValueAt(i,1) == null)
              || ((String) this.roomDescription.extraDescriptionPanel.table.getValueAt(i,0)).trim().equals("")
              || ((String) this.roomDescription.extraDescriptionPanel.table.getValueAt(i,1)).trim().equals("")
            )
        {
        }
        else
        {
            String keywords = new String((String) this.roomDescription.extraDescriptionPanel.table.getValueAt(i,0));
            String desc = new String((String) this.roomDescription.extraDescriptionPanel.table.getValueAt(i,1));
            if (!keywords.trim().equals(""))
               anatoliaRoom.extras.add(new Pair(keywords, desc));
        }
      }

      for(int i=0 ; i<this.flagsPanel.flags.length ; ++i)
         anatoliaRoom.flags[i] = this.flagsPanel.flags[i].isSelected();

      anatoliaRoom.randomUpTo = (byte) this.flagsPanel.randomUpToBox.getSelectedIndex();

      for (byte i=0; i<this.sectorPanel.sectors.length ; ++i)
      {
         if (this.sectorPanel.sectors[i].isSelected())
         {
            anatoliaRoom.sectorType = i;
            break;
         }
      }

      anatoliaRoom.HPRecovery = Math.max(1, HPMPRecoveryPanel.hpSlider.getValue());
      anatoliaRoom.MPRecovery = Math.max(1,HPMPRecoveryPanel.mpSlider.getValue());

      //doors
      for(int i=0 ; i<6 ; ++i)
      {
         if (anatoliaRoom.doors[i] != null)
         {
            anatoliaRoom.doors[i].doorType = this.exits.theExits[i].doorType.isSelected() ?
                                            AnatoliaConsts.EXIT_WITH_DOOR :
                                            AnatoliaConsts.EXIT_WITHOUT_DOOR;

            anatoliaRoom.doors[i].doorState = (byte) this.exits.theExits[i].doorState.getSelectedIndex();
            anatoliaRoom.doors[i].doorSize = (byte) this.exits.theExits[i].doorSize.getSelectedIndex();

            AnatoliaObject[] keys = areaMap.theObjects.getAllKeyObjects();
            int choice = this.exits.theExits[i].keyBox.getSelectedIndex();
            if (choice == 0)
               anatoliaRoom.doors[i].keyVnum = 0;
            else
               anatoliaRoom.doors[i].keyVnum = ((AnatoliaObject) this.exits.theExits[i].keyBox.getSelectedItem()).getVnum();

            anatoliaRoom.doors[i].extraDescription =
              new String(this.exits.theExits[i].description.getText());
            anatoliaRoom.doors[i].doorKeyword =
              new String(this.exits.theExits[i].doorKeyword.getText());
            for(int j=0 ; j<this.exits.theExits[i].doorResets.length ; ++j)
               anatoliaRoom.doors[i].doorResetFlags[j]=this.exits.theExits[i].doorResets[j].isSelected();

            //the OTHER side of the door should be set the same way
            anatoliaRoom.doors[i].autoSetSymetricDoor = this.exits.theExits[i].autoSetSymetricDoor.isSelected();
            Room goingTo = (anatoliaRoom.myRoom.connectingToRooms[i] != null)         ?
                           (Room) anatoliaRoom.myRoom.connectingToRooms[i].getFirst() :
                           null;
            if (goingTo != null)
            {
              //looking for the door going here
              int  found = -1;
              for(int j=0 ; j<goingTo.connectingToRooms.length ; ++j)
              {
                if (goingTo.connectingToRooms[j] != null)
                {
                  if ( ((Room) goingTo.connectingToRooms[j].getFirst()) == anatoliaRoom.myRoom)
                  {
                    found = j;
                    break;
                  }
                }
              }
              if (found >= 0)
              {
                AnatoliaRoom symetricRoom = goingTo.anatoliaRoom;
                boolean thisSide  = anatoliaRoom.doors[i].autoSetSymetricDoor;
                boolean otherSide = symetricRoom.doors[found].autoSetSymetricDoor;
                if (thisSide && otherSide)
                {
                  symetricRoom.doors[found].doorKeyword = anatoliaRoom.doors[i].doorKeyword;
                  symetricRoom.doors[found].doorResetFlags = anatoliaRoom.doors[i].doorResetFlags;
                  symetricRoom.doors[found].doorState = anatoliaRoom.doors[i].doorState;
                  symetricRoom.doors[found].doorSize = anatoliaRoom.doors[i].doorSize;
                  symetricRoom.doors[found].doorType = anatoliaRoom.doors[i].doorType;
                  symetricRoom.doors[found].extraDescription = anatoliaRoom.doors[i].extraDescription;
                  symetricRoom.doors[found].keyVnum = anatoliaRoom.doors[i].keyVnum;
                }
                else if(!thisSide && !otherSide)
                {
                  //nothing to do
                }
                else if(thisSide && !otherSide)
                {
                  int choice1 = JOptionPane.showConfirmDialog(this,
                                    "The other side of the door is not configured"+"\n"+
                                    "to match automatically."+"\n"+
                                    "Do you want to do make it match anyway ?","Warning",
                                    JOptionPane.YES_NO_OPTION,
                                    JOptionPane.WARNING_MESSAGE);
                  if (choice1 == JOptionPane.YES_OPTION)
                  {
                    symetricRoom.doors[found].autoSetSymetricDoor = true;
                    symetricRoom.doors[found].doorKeyword = anatoliaRoom.doors[i].doorKeyword;
                    symetricRoom.doors[found].doorResetFlags = anatoliaRoom.doors[i].doorResetFlags;
                    symetricRoom.doors[found].doorState = anatoliaRoom.doors[i].doorState;
                    symetricRoom.doors[found].doorSize = anatoliaRoom.doors[i].doorSize;
                    symetricRoom.doors[found].doorType = anatoliaRoom.doors[i].doorType;
                    symetricRoom.doors[found].extraDescription = anatoliaRoom.doors[i].extraDescription;
                    symetricRoom.doors[found].keyVnum = anatoliaRoom.doors[i].keyVnum;
                  }
                }
                else if(!thisSide && otherSide)
                {
                  int choice2 = JOptionPane.showConfirmDialog(this,
                                    "The other side of the door is configured"+"\n"+
                                    "to match automatically."+"\n"+
                                    "Do you want to still want to keep your changes"+"\n"+
                                    "and unsynchronise the two sides of the door ?",
                                    "Warning",
                                    JOptionPane.YES_NO_OPTION,
                                    JOptionPane.WARNING_MESSAGE);
                  if (choice2 == JOptionPane.NO_OPTION)
                  {
                    anatoliaRoom.doors[i].doorKeyword = symetricRoom.doors[found].doorKeyword;
                    anatoliaRoom.doors[i].doorResetFlags = symetricRoom.doors[found].doorResetFlags;
                    anatoliaRoom.doors[i].doorState = symetricRoom.doors[found].doorState;
                    anatoliaRoom.doors[i].doorSize = symetricRoom.doors[found].doorSize;
                    anatoliaRoom.doors[i].doorType = symetricRoom.doors[found].doorType;
                    anatoliaRoom.doors[i].extraDescription = symetricRoom.doors[found].extraDescription;
                    anatoliaRoom.doors[i].keyVnum = symetricRoom.doors[found].keyVnum;
                  }
                } /*end different cases of auto set opposite side */
              } /*else found symetric door */
            } /*else found other room */
         } /*end if door */
      } /*end for each door */

      this.areaMap.objectTreeDialog.repaint();
      return true;
   }
}

class RoomNumbering extends JPanel
{
   JCheckBox outsideRoom = new JCheckBox("Room does NOT belong to area", false);
   JTextField vnum = new JTextField("99999");
   JCheckBox autoVnum = new JCheckBox("automatic", true);
   public RoomNumbering()
   {
      Box box = Box.createVerticalBox();
      box.add(outsideRoom);

      JPanel vnumPane = new JPanel(new FlowLayout(FlowLayout.CENTER, 10, 0));
      vnumPane.add(new JLabel("Vnum :"));
      vnumPane.add(vnum);
      vnumPane.add(autoVnum);
      vnum.setEnabled(!autoVnum.isSelected());

      box.add(vnumPane);
      add(box);
   }
}

class RoomDescription extends JPanel
{
   JTextField roomName = new JTextField(40);
   JTextArea roomDesc = new JTextArea(6,40);
   ExtraDescriptions extraDescriptionPanel = new ExtraDescriptions();

   public RoomDescription()
   {
      Box box = Box.createVerticalBox();
      box.add(new JLabel("Room name (a few words)"));
      box.add(roomName);
      box.add(new JLabel("Room description (a few lines)"));
      JScrollPane j = new JScrollPane(roomDesc,
                          JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                          JScrollPane.HORIZONTAL_SCROLLBAR_NEVER
                         );
      box.add(j);
      box.add(extraDescriptionPanel);
      roomDesc.setLineWrap(true);
      add(box);
   }
}

class Flags extends JPanel implements ActionListener
{
    private JPanel flagsPanel = new JPanel();
    JCheckBox[] flags = new JCheckBox[AnatoliaConsts.roomFlagsTexts.length];
    boolean[] sameFlags = new boolean[AnatoliaConsts.roomFlagsTexts.length];
    JComboBox randomUpToBox = new JComboBox(AnatoliaConsts.directionTexts);
    boolean sameRandomUpTo = true;

    private ButtonGroupMaybeNone buttonGroup1 = new ButtonGroupMaybeNone();
    private ButtonGroupMaybeNone buttonGroup2 = new ButtonGroupMaybeNone();

    public Flags()
    {
       //adds check boxes
       setLayout(new GridLayout(5,4));
       for(byte flag=0 ; flag<flags.length; ++flag)
       {
          sameFlags[flag] = true;
          flags[flag] = new JCheckBox(AnatoliaConsts.roomFlagsTexts[flag].getName());
          flags[flag].setToolTipText(AnatoliaConsts.roomFlagsTexts[flag].getToolTipText());
          flags[flag].addActionListener(this);
          add(flags[flag]);
       }
       JPanel randomExits = new JPanel(new FlowLayout());
       randomExits.add(new JLabel("Up to"));
       randomExits.add(randomUpToBox);
       add(randomExits);
       flags[AnatoliaConsts.ROOM_FLAG_RANDOM].addActionListener(this);
       randomUpToBox.setEnabled(flags[AnatoliaConsts.ROOM_FLAG_RANDOM].isSelected());
       randomUpToBox.setToolTipText("Sets up the \"maximum\" door to randomize (from North to-???)");
       randomUpToBox.addActionListener(this);
       setBorder(new TitledBorder("Flags"));

       buttonGroup1.add(flags[AnatoliaConsts.ROOM_FLAG_SOLITARY]);
       buttonGroup1.add(flags[AnatoliaConsts.ROOM_FLAG_PRIVATE]);

       buttonGroup2.add(flags[AnatoliaConsts.ROOM_FLAG_NEWBIES_ONLY]);
       buttonGroup2.add(flags[AnatoliaConsts.ROOM_FLAG_HEROES_ONLY]);
       buttonGroup2.add(flags[AnatoliaConsts.ROOM_FLAG_GODS_ONLY]);
       buttonGroup2.add(flags[AnatoliaConsts.ROOM_FLAG_IMP_ONLY]);
    }

    public void actionPerformed(ActionEvent e)
    {
       if (e.getSource() == flags[AnatoliaConsts.ROOM_FLAG_RANDOM])
       {
          randomUpToBox.setEnabled(flags[AnatoliaConsts.ROOM_FLAG_RANDOM].isSelected());
       }
       if (e.getSource() == randomUpToBox)
       {
          sameRandomUpTo = true;
          randomUpToBox.setForeground(Color.black);
       }
       else
       {
          ((JCheckBox) e.getSource()).setForeground(Color.black);
          //finds the flag actionned
          int actionnedFlag = -1;
          for(int i=0 ; i<flags.length ; ++i)
          {
             if (e.getSource() == flags[i])
             {
                actionnedFlag = i;
                break;
             }
          }
          sameFlags[actionnedFlag] = true;
       }
    }
}

class SectorTypes extends JPanel implements ActionListener
{
    ButtonGroup sectorGroup = new ButtonGroup();
    JRadioButton[] sectors = new JRadioButton[AnatoliaConsts.roomSectorTexts.length];
    boolean sameSectorType = true;

    public SectorTypes()
    {
       //adds radio buttons
       setLayout(new GridLayout(3,4));
       for(byte sector=0 ; sector<sectors.length; ++sector)
       {
          sectors[sector] = new JRadioButton(AnatoliaConsts.roomSectorTexts[sector].getName());
          if (sector != AnatoliaConsts.ROOM_SECTOR_UNUSED)
          {
             sectors[sector].setToolTipText(AnatoliaConsts.roomSectorTexts[sector].getToolTipText());
             add(sectors[sector]);
             sectorGroup.add(sectors[sector]);
             sectors[sector].addActionListener(this);
          }
       }
       sectors[AnatoliaConsts.ROOM_SECTOR_INSIDE].setEnabled(true);
       setBorder(new TitledBorder("Sector type"));
    }

    public void actionPerformed(ActionEvent e)
    {
       sameSectorType = true;
       for(int i=0 ; i<sectors.length ; ++i)
          sectors[i].setForeground(Color.black);
    }
}

class HPMPRecovery extends JPanel implements ChangeListener
{
   JSlider  hpSlider = new JSlider(JSlider.HORIZONTAL,0,200,100);
   JSlider  mpSlider = new JSlider(JSlider.HORIZONTAL,0,200,100);
   JLabel hpLabel = new JLabel("HP recovery rate : "+ hpSlider.getValue()+"%");
   JLabel mpLabel = new JLabel("MP recovery rate : "+ mpSlider.getValue()+"%");
   boolean sameHPRecovery = true;
   boolean sameMPRecovery = true;

    public HPMPRecovery()
    {
       setLayout(new FlowLayout(FlowLayout.CENTER, 30, 0));

       Hashtable labels = new Hashtable();
       for(int i=0 ; i<=200 ; i+=50)
       {
          labels.put(new Integer(i),new JLabel(Integer.toString(i)));
       }
       JPanel hpPanel = new JPanel(new GridLayout(2,1));
         hpPanel.add(hpLabel);
         hpSlider.setLabelTable(labels);
         hpSlider.setPaintLabels(true);
         hpSlider.setMajorTickSpacing(10);
         hpSlider.setMinorTickSpacing(5);
         hpSlider.setPaintTicks(true);
         hpSlider.setSnapToTicks(true);
         hpSlider.addChangeListener(this);
         hpPanel.add(hpSlider);
       add(hpPanel);

       JPanel mpPanel = new JPanel(new GridLayout(2,1));
         mpPanel.add(mpLabel);
         mpSlider.setLabelTable(labels);
         mpSlider.setPaintLabels(true);
         mpSlider.setMajorTickSpacing(10);
         mpSlider.setMinorTickSpacing(5);
         mpSlider.setPaintTicks(true);
         mpSlider.setSnapToTicks(true);
         mpSlider.addChangeListener(this);
         mpPanel.add(mpSlider);
       add(mpPanel);

       setBorder(new TitledBorder("HP and MP recovery rates"));
    }
   public void stateChanged(ChangeEvent e)
   {
      if (e.getSource() == hpSlider)
      {
         hpLabel.setForeground(Color.black);
         hpLabel.setText("HP recovery rate : "+hpSlider.getValue()+"%");
      }
      else if (e.getSource() == mpSlider)
      {
         mpLabel.setForeground(Color.black);
         mpLabel.setText("MP recovery rate : "+mpSlider.getValue()+"%");
      }
   }
}

class Exit extends JPanel implements ActionListener
{
   private final static String[] doorTypeTexts = {"No door", "Door"};
   JTextField description = new JTextField(40);
   JTextField doorKeyword = new JTextField(20);

   JComboBox doorState = new JComboBox(AnatoliaConsts.getDoorStateTexts());
   JComboBox doorSize = new JComboBox(AnatoliaConsts.getDoorSizeTexts());
   JCheckBox doorType = new JCheckBox("There is a door");
   JCheckBox autoSetSymetricDoor = new JCheckBox("Auto set other side door");
   JComboBox keyBox = new JComboBox();
   private ButtonGroup doorButtonGroup1 = new ButtonGroup();
   private ButtonGroupMaybeNone doorButtonGroup2 = new ButtonGroupMaybeNone();
   private ButtonGroupMaybeNone doorButtonGroup3 = new ButtonGroupMaybeNone();
   private ButtonGroupMaybeNone doorButtonGroup4 = new ButtonGroupMaybeNone();

   JCheckBox[] doorResets = new JCheckBox[AnatoliaConsts.doorResetsTexts.length];

   public Exit()
   {
      setLayout(new GridBagLayout());
      GridBagConstraints constraint = new GridBagConstraints();

      constraint.gridx=0;constraint.gridy=0;
      constraint.gridwidth=4;constraint.gridheight=1;
      constraint.weightx=100;constraint.weighty=15;
      Box descriptionBox = Box.createVerticalBox();
      descriptionBox.add(new JLabel("Description of what is seen in that direction (optional)"));
      descriptionBox.add(description);
      add(descriptionBox, constraint);

      constraint.gridx=0;constraint.gridy=1;
      constraint.gridwidth=2;constraint.gridheight=1;
      constraint.weightx=50;constraint.weighty=15;
      doorType.addActionListener(this);
      doorType.setSelected(false);
      add(doorType, constraint);

      constraint.gridx=1;constraint.gridy=1;
      constraint.gridwidth=2;constraint.gridheight=1;
      constraint.weightx=50;constraint.weighty=15;
      autoSetSymetricDoor.addActionListener(this);
      autoSetSymetricDoor.setToolTipText("The other side of the door is automatically set with the same parameters");
      autoSetSymetricDoor.setSelected(true);
      add(autoSetSymetricDoor, constraint);


      JPanel doorKeywordPane = new JPanel(new GridLayout(2,1));
      constraint.gridx=0;constraint.gridy=2;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=25;constraint.weighty=15;
      doorKeywordPane.add(new JLabel("Door keyword (opt.):"));
      doorKeywordPane.add(doorKeyword);
      doorKeyword.setEnabled(doorType.isSelected());
      add(doorKeywordPane, constraint);


      JPanel doorStatePane = new JPanel(new GridLayout(2,1));
      constraint.gridx=1;constraint.gridy=2;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=25;constraint.weighty=15;
      doorStatePane.add(new JLabel("Door state :"));
      doorStatePane.add(doorState);
      doorState.setEnabled(doorType.isSelected());
      add(doorStatePane, constraint);

      JPanel keyPanel = new JPanel(new GridLayout(2,1));
      keyPanel.add(new JLabel("key"), constraint);
      keyPanel.add(keyBox);
      constraint.gridx=2;constraint.gridy=2;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=25;constraint.weighty=15;
      keyBox.setEnabled(doorType.isSelected());
      add(keyPanel, constraint);


      JPanel doorSizePane = new JPanel(new GridLayout(2,1));
      constraint.gridx=3;constraint.gridy=2;
      constraint.gridwidth=1;constraint.gridheight=1;
      constraint.weightx=25;constraint.weighty=15;
      doorSizePane.add(new JLabel("Door size :"));
      doorSizePane.add(doorSize);
      add(doorSizePane, constraint);
      doorSize.setSelectedIndex(6); //GARGANTUAN by default
      doorSize.setEnabled(true);//doorType.isSelected());

      JPanel doorResetsPane = new JPanel(new GridLayout(AnatoliaConsts.doorResetsTexts.length, 1));
      for(int i=0 ; i<AnatoliaConsts.doorResetsTexts.length ; ++i)
      {
         doorResets[i] = new JCheckBox(AnatoliaConsts.doorResetsTexts[i].getName());
         doorResetsPane.add(doorResets[i]);
      }
      constraint.gridx=0;constraint.gridy=3;
      constraint.gridwidth=3;constraint.gridheight=1;
      constraint.weightx=100;constraint.weighty=45;
      doorResetsPane.setBorder(new TitledBorder("Resets"));
      add(doorResetsPane, constraint);

      doorResets[AnatoliaConsts.DOOR_RESETS_OPENED].setSelected(true);
      doorButtonGroup1.add(doorResets[AnatoliaConsts.DOOR_RESETS_OPENED]);
      doorButtonGroup1.add(doorResets[AnatoliaConsts.DOOR_RESETS_CLOSED]);
      doorButtonGroup1.add(doorResets[AnatoliaConsts.DOOR_RESETS_LOCKED]);
      doorButtonGroup2.add(doorResets[AnatoliaConsts.DOOR_EASY_TO_PICK]);
      doorButtonGroup2.add(doorResets[AnatoliaConsts.DOOR_HARD_TO_PICK]);
      doorButtonGroup2.add(doorResets[AnatoliaConsts.DOOR_INFURIATING_TO_PICK]);
      doorButtonGroup2.add(doorResets[AnatoliaConsts.DOOR_CANNOT_BE_PICKED]);
      doorButtonGroup3.add(doorResets[AnatoliaConsts.DOOR_CANNOT_BE_CLOSED]);
      doorButtonGroup3.add(doorResets[AnatoliaConsts.DOOR_RESETS_CLOSED]);
      doorButtonGroup4.add(doorResets[AnatoliaConsts.DOOR_CANNOT_BE_LOCKED]);
      doorButtonGroup4.add(doorResets[AnatoliaConsts.DOOR_RESETS_LOCKED]);
   }

   public void setEnabled(boolean state)
   {
      doorType.setEnabled(state);
      autoSetSymetricDoor.setEnabled(state);
      description.setEnabled(state);// && doorType.isSelected());
      doorState.setEnabled(state && doorType.isSelected());
      doorSize.setEnabled(state && doorType.isSelected());
      keyBox.setEditable(state && doorType.isSelected());
      for(int i=0 ; i<AnatoliaConsts.doorResetsTexts.length ; ++i)
         doorResets[i].setEnabled(state && doorType.isSelected());
   }

   public void actionPerformed(ActionEvent e)
   {
      if (e.getSource() == doorType)
      {
        setDoorEnabled(doorType.isSelected());
      }
   }

   public void setDoorEnabled(boolean state)
   {
      autoSetSymetricDoor.setEnabled(state);
      doorKeyword.setEnabled(state);
      doorState.setEnabled(state);
      doorSize.setEnabled(state);
      keyBox.setEnabled(state);
      for(int i=0 ; i<AnatoliaConsts.doorResetsTexts.length ; ++i)
         doorResets[i].setEnabled(state);
   }

}

class Exits extends JPanel
{
   JTabbedPane tabbedPane = new JTabbedPane();
   Exit[] theExits = new Exit[6];

   public Exits()
   {
      setLayout(new GridLayout(1,1));
      for(byte door=0; door<6; ++door)
      {
         theExits[door] = new Exit();
         tabbedPane.addTab(AnatoliaConsts.directionTexts[door], theExits[door]);
      }
      add(tabbedPane);
   }
}

class ExtraDescriptions extends JPanel
{
   JTable table = new JTable(8,2);

   public ExtraDescriptions()
   {
      for(int i=0 ; i<table.getRowCount() ; ++i)
      {
         table.setValueAt("", i, 0);
         table.setValueAt("", i, 1);
      }

      TableColumn column = null;
      TableColumnModel columnModel = table.getColumnModel();

      column = columnModel.getColumn(0);
      column.setHeaderValue("keywords");
      column.setPreferredWidth(200);
      column = columnModel.getColumn(1);
      column.setHeaderValue("Description");
      column.setPreferredWidth(800);
      add(new JScrollPane(table));
      table.setPreferredScrollableViewportSize(new Dimension(400, 100));

      setBorder(new TitledBorder("Details of architecture"));
   }
}