/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import java.awt.*;
import javax.swing.JToggleButton;
import javax.swing.tree.*;
import java.util.*; //for linkedlist...

public class AreaBuilderConsts
{
   public final static byte INSTANCE_TYPE_MOBILE = 1;
   public final static byte INSTANCE_TYPE_OBJECT = 2;

   public AreaBuilderConsts()
   {
   }

   public static AnatoliaRoom findAnatoliaRoomOfVnum(LinkedList outsideRooms, int vnum)
   {
     if (outsideRooms == null) return null;
     Iterator it = outsideRooms.iterator();
     AnatoliaRoom room = null;
     while(it.hasNext())
     {
       room = (AnatoliaRoom) it.next();
       if (room.getVnum() == vnum) return room;
     }
     return null;
   }

   public static void stringToSelection(String string, FlagStructure[] flagStructures, JToggleButton[] selection)
   {
     if (string == null) return;
     char c;
     int found;
     for(int i=0 ; i<selection.length ; ++i)
     {
        selection[i].setSelected(false);
     }

     for(int i=0 ; i<string.length() ; ++i)
     {
        c = string.charAt(i);
        found = -1;
        for(int j=0 ; j<flagStructures.length ; ++j)
        {
           if (flagStructures[i] == null) continue;

           if ((flagStructures[j].getCode() == null) ||
               (flagStructures[j].getCode().trim().equals("")))
              continue;

           if (flagStructures[j].getCode().charAt(0) == c)
           {
              found = j;
              break;
           }
        }
        if (found >= 0)
        {
           selection[found].setSelected(true);
        }
     }
  }

   public static void stringToSelection(String string, FlagStructure[] flagStructures, boolean[] selection)
   {
     if (string == null) return;
     char c;
     int found;
     for(int i=0 ; i<selection.length ; ++i)
     {
        selection[i] = false;
     }

     for(int i=0 ; i<string.length() ; ++i)
     {
        c = string.charAt(i);
        found = -1;
        for(int j=0 ; j<flagStructures.length ; ++j)
        {
           if (flagStructures[j] == null) continue;

           if ((flagStructures[j].getCode() == null) ||
               (flagStructures[j].getCode().trim().equals("")))
              continue;
           if (flagStructures[j].getCode().charAt(0) == c)
           {
              found = j;
              break;
           }
        }
        if (found >= 0)
        {
           selection[found] = true;
        }
     }
  }

  public static String selectionToString(FlagStructure[] flagStructures, JToggleButton[] selection)
  {
     String tempString = "";
     for(int i=0 ; i<selection.length ; ++i)
     {
        if (selection[i].isSelected() && (flagStructures[i] != null)) tempString += flagStructures[i].getCode();
     }
     if (tempString.trim().equals(""))
       return "0";
    else
       return tempString;
  }

  public static int findInTab(Object[] tab, Object searched)
  {
     for(int i=0 ; i<tab.length ; ++i)
     {
        if ((tab[i] != null) && (tab[i].equals(searched))) return i;
     }
     return -1;
  }

  public static AnatoliaRoom getRoomNode(DefaultMutableTreeNode node)
  {
    try
    {
       AnatoliaRoom room = (AnatoliaRoom) node;
       return room;
    }
    catch(Exception e) {return null;}
  }

  public static InstanceOfMobile getMobileNode(DefaultMutableTreeNode node)
  {
    try
    {
       InstanceOfMobile mobile = (InstanceOfMobile) node;
       return mobile;
    }
    catch(Exception e) {return null; }
  }

  public static InstanceOfObject getObjectNode(DefaultMutableTreeNode node)
  {
    try
    {
       InstanceOfObject object = (InstanceOfObject) node;
       return object;
    }
    catch(Exception e) {return null;}
  }

  public static String interpretFlags(String flags)
  {
    final String[] letters =
      {"A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q",
       "R","S","T","U","V","W","X","Y","Z","a","b","c","d","e","f","g","h","i",
       "k","l","m","n","o","p","q","r","s","t","u","v","w","x","y","z"};
    if (    flags.equals("0")
         ||
         ((flags.indexOf("1")<0)
          && (flags.indexOf("2")<0)
          && (flags.indexOf("3")<0)
          && (flags.indexOf("4")<0)
          && (flags.indexOf("5")<0)
          && (flags.indexOf("6")<0)
          && (flags.indexOf("7")<0)
          && (flags.indexOf("8")<0)
          && (flags.indexOf("9")<0)
         )
        )
       {
         return flags;
       }
       else //flags are 2|8192, etc...
       {
          StringTokenizer tokens2 = new StringTokenizer(flags,"|");
          String result = "";
          while (tokens2.hasMoreTokens())
          {
             int nb = MyInteger.parseInt(tokens2.nextToken());
             nb = (int) (Math.log(nb)/Math.log(2));
             if (nb >= letters.length) nb = letters.length-1;
             if (nb >=0) result = result + letters[nb];
          }
          if (result.trim().equals("")) result = "0";
          return result;
       }
   }
}