/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*; //for archivation
import java.util.*; //for stringtokenizer

public class DiceRoll extends JPanel implements ActionListener, Serializable
{
  protected int nbDices;
  protected int nbFaces;
  protected int nbBonus;
  protected JTextField nbDicesTextField = new JTextField(3);
  protected JTextField nbFacesTextField = new JTextField(3);
  protected JTextField nbBonusTextField = new JTextField(4);
  public DiceRoll()
  {
     super();
     this.nbDices = 0;
     this.nbFaces = 0;
     this.nbBonus = 0;
     build();
  }
  public DiceRoll(int nbDices, int nbFaces, int nbBonus)
  {
     super();
     this.nbDices = nbDices;
     this.nbFaces = nbFaces;
     this.nbBonus  = nbBonus;
     build();
  }
  public DiceRoll(String xdypz)
  {
     super();
     this.nbDices = 0;
     this.nbFaces = 0;
     this.nbBonus = 0;
     StringTokenizer tokens = new StringTokenizer(xdypz,"d+-");
     if (tokens.hasMoreTokens())
       this.nbDices = MyInteger.parseInt(tokens.nextToken());
     if (tokens.hasMoreTokens())
       this.nbFaces = MyInteger.parseInt(tokens.nextToken());
     if (tokens.hasMoreTokens())
       this.nbBonus = MyInteger.parseInt(tokens.nextToken());
     build();
  }

  public Object clone()
  {
     return new DiceRoll(nbDices,nbFaces,nbBonus);
  }

  public void setNbDices(int nbDices) {this.nbDices = nbDices; this.nbDicesTextField.setText(Integer.toString(nbDices));}
  public void setNbFaces(int nbFaces) {this.nbFaces = nbFaces; this.nbFacesTextField.setText(Integer.toString(nbFaces));}
  public void setNbBonus(int nbBonus) {this.nbBonus = nbBonus; this.nbBonusTextField.setText(Integer.toString(nbBonus));}
  public void setAll(int nbDices, int nbFaces, int nbBonus) {setNbDices(nbDices);setNbFaces(nbFaces);setNbBonus(nbBonus);}
  public void setDiceRoll(DiceRoll other) {setAll(other.nbDices, other.nbFaces, other.nbBonus);}

  public int getNbDices() {return nbDices;}
  public int getNbFaces() {return nbFaces;}
  public int getNbBonus() {return nbBonus;}

  private void build()
  {
     setLayout(new FlowLayout());
     add(nbDicesTextField);
     nbDicesTextField.addActionListener(this);
     add(new JLabel("d"));
     add(nbFacesTextField);
     nbFacesTextField.addActionListener(this);
     add(new JLabel("+"));
     add(nbBonusTextField);
     nbBonusTextField.addActionListener(this);
  }

  public String toString()
  {
     return nbDices+"d"+nbFaces+"+"+nbBonus;
  }

  public void actionPerformed(ActionEvent e)
  {
     if (e.getSource() == nbDicesTextField)
        nbDices = MyInteger.parseInt(nbDicesTextField.getText());
     else if (e.getSource() == nbFacesTextField)
        nbFaces = MyInteger.parseInt(nbFacesTextField.getText());
     else if (e.getSource() == nbBonusTextField)
        nbBonus = MyInteger.parseInt(nbBonusTextField.getText());
  }

   public void writeObject(ObjectOutputStream output) throws IOException
   {
      output.writeInt(nbDices);
      output.writeInt(nbFaces);
      output.writeInt(nbBonus);
   }

   public void readObject(ObjectInputStream input) throws IOException, ClassNotFoundException
   {
      nbDices = input.readInt();
      nbFaces = input.readInt();
      nbBonus = input.readInt();
   }
}