/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import java.util.*; //for iterator, lists,...
import java.io.*; //for archivation

public class ObjectTree implements Serializable, TreeSelectionListener
{
  AreaMap areaMap = null;
  JTree treeOfObjects = null;
  DefaultMutableTreeNode root = null;
  DefaultTreeModel treeModel = null;

  public ObjectTree(AreaMap areaMap)
  {
     this.areaMap = areaMap;
     root = new DefaultMutableTreeNode("Area");
     treeModel = new DefaultTreeModel(root);
     treeOfObjects = new JTree(treeModel);
     treeOfObjects.setCellRenderer(new MyTreeCellRenderer());
     treeOfObjects.setRootVisible(false);
     treeOfObjects.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
     treeOfObjects.addTreeSelectionListener(this);
  }

  public void addNode(DefaultMutableTreeNode treeNode)
  {
     addNode(treeNode, null);
  }
  public void addNode(DefaultMutableTreeNode treeNode, DefaultMutableTreeNode parent)
  {
     if (parent == null) parent=root;
     treeModel.insertNodeInto(treeNode, parent, parent.getChildCount());
     treeOfObjects.fireTreeExpanded(new TreePath(treeNode.getPath()));
     treeOfObjects.scrollPathToVisible(new TreePath(treeNode.getPath()));
     areaMap.objectTreeDialog.pack();
  }
  public void addNode(DefaultMutableTreeNode treeNode, DefaultMutableTreeNode parent, int rank)
  {
     if (parent == null) parent=root;
     treeModel.insertNodeInto(treeNode, parent, rank);
     treeOfObjects.fireTreeExpanded(new TreePath(treeNode.getPath()));
     treeOfObjects.scrollPathToVisible(new TreePath(treeNode.getPath()));
     areaMap.objectTreeDialog.pack();
  }

  //returns the former child rank
  public int removeNode(DefaultMutableTreeNode node)
  {
     int childRank = node.getParent().getIndex(node);
     treeModel.removeNodeFromParent(node);
     return childRank;
  }

  public void delTree(DefaultMutableTreeNode aRoot, UndoableActionStack actionStack)
  {
     if (aRoot == null) return;

     while(treeModel.getChildCount(aRoot)>0)
     {
        delTree((DefaultMutableTreeNode) treeModel.getChild(aRoot, treeModel.getChildCount(aRoot)-1), actionStack);
     }

     if (aRoot.getParent() == null) return;

     if (actionStack != null)
     {
       try
       {
         InstanceOfMobile mobile = (InstanceOfMobile) aRoot;
         actionStack.doAction(
           new UndoableActionObjectTreeInstanceOfMobileRemove(this, mobile, UndoableAction.AUTOMATIC_ACTION)
         );
       }
       catch(Exception e) {}
       try
       {
         InstanceOfObject object = (InstanceOfObject) aRoot;
         actionStack.doAction(
           new UndoableActionObjectTreeInstanceOfObjectRemove(this, object, UndoableAction.AUTOMATIC_ACTION)
         );
       }
       catch(Exception e) {}
       actionStack.doAction(
         new UndoableActionObjectTreeNodeRemove(this, aRoot, UndoableAction.AUTOMATIC_ACTION)
       );
      }
     else
     {
       treeModel.removeNodeFromParent(aRoot);
     }
  }

  public void clear()
  {
    if ((areaMap == null) || (areaMap.mainScreen == null))
      delTree(root, null);
    else
      delTree(root, areaMap.mainScreen.actionStack);
  }

  public void writeTree(AnatoliaRoom aRoot,ObjectOutputStream output) throws IOException
  {
     output.writeInt(aRoot.getChildCount());
     for(int i=0 ; i<aRoot.getChildCount() ; ++i)
     {
        writeTreeFrom((AbstractInstance) aRoot.getChildAt(i), output);
     }
  }
  public void writeTreeFrom(AbstractInstance aRoot,ObjectOutputStream output) throws IOException
  {
     output.writeByte(aRoot.getType());
     aRoot.writeObject(output);
     output.writeInt(aRoot.getChildCount());
     for(int i=0 ; i<aRoot.getChildCount() ; ++i)
     {
        writeTreeFrom((AbstractInstance) aRoot.getChildAt(i), output);
     }
  }
  public void readTree(AnatoliaRoom aRoot,ObjectInputStream input) throws IOException, ClassNotFoundException
  {
     int tempSize = input.readInt();
     for(int i=0 ; i<tempSize ; ++i)
     {
        readTreeFrom(aRoot, input);
     }
  }

  public void readTreeFrom(DefaultMutableTreeNode aRoot, ObjectInputStream input) throws IOException, ClassNotFoundException
  {
    AbstractInstance instance = null;
    byte type = input.readByte();
    if (type == AreaBuilderConsts.INSTANCE_TYPE_MOBILE)
      instance = new InstanceOfMobile(null);
    else if (type == AreaBuilderConsts.INSTANCE_TYPE_OBJECT)
      instance = new InstanceOfObject(null, 0);
    instance.readObject(input);
    instance.establishLinks();
    this.addNode(instance, aRoot);

    int tempSize = input.readInt();
    for(int i=0 ; i<tempSize ; ++i)
    {
       readTreeFrom(instance, input);
    }
  }

   public void writeObject(ObjectOutputStream output) throws IOException
   {
      //adds the rooms
      Iterator it = areaMap.theRooms.getIterator();
      AnatoliaRoom anatoliaRoom = null;
      while(it.hasNext())
      {
        anatoliaRoom = ((Room) it.next()).anatoliaRoom;
        writeTree(anatoliaRoom, output);
      }
   }

   public void readObject(ObjectInputStream input) throws IOException, ClassNotFoundException
   {
      clear();

      Iterator it = areaMap.theRooms.getIterator();
      AnatoliaRoom anatoliaRoom = null;
      while(it.hasNext())
      {
        anatoliaRoom = ((Room) it.next()).anatoliaRoom;
        this.addNode(anatoliaRoom);
        readTree(anatoliaRoom, input);
      }
   }

   public void valueChanged(TreeSelectionEvent e)
   {
     AnatoliaRoom room = null;
     for(int i=0 ; (room == null) && (i<e.getPath().getPath().length); ++i)
     {
       try
       {
          room = (AnatoliaRoom) e.getPath().getPathComponent(i);
       }
       catch(Exception ex) {}
     }

     if (room != null) this.areaMap.selectedRooms.resetSelection(room.myRoom);
     areaMap.repaint();
   }
}