/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.util.*; //Iterator

public class ObjectTreeDialog extends JDialog implements WindowListener, ActionListener, TreeSelectionListener
{
  ObjectTree objectTree = null;
  JTree bufferTree = null;
  DefaultTreeModel bufferTreeModel = null;
  UndoableActionStack actionStack = new UndoableActionStack();

  JScrollPane scrollPane = null;
  JMenuBar menu = new JMenuBar();
    JMenu menuAdd = new JMenu("Add");
      JMenuItem menuNewMobile = new JMenuItem("Create new Mobile");
      JMenuItem menuAddMobileInstance = new JMenuItem("Add mobile instance");
      JMenuItem menuAddExternalMobileInstance = new JMenuItem("Add external mobile instance");
      JMenuItem menuNewObject = new JMenuItem("Create new Object");
      JMenuItem menuAddObjectInstance = new JMenuItem("Add object instance");
      JMenuItem menuAddExternalObjectInstance = new JMenuItem("Add external object instance");
    JMenu menuRemove = new JMenu("Remove");
      JMenuItem menuRemoveMobile = new JMenuItem("Remove a mobile");
      JMenuItem menuRemoveObject = new JMenuItem("Remove an object");
      JMenuItem menuRemoveInstance = new JMenuItem("Remove selected instance");
     JMenu menuEdition = new JMenu("Edition");
      JMenuItem menuEditionUndo = new JMenuItem("Undo");
      JMenuItem menuEditionRedo = new JMenuItem("Redo");
      JMenuItem menuEditionCopyInstance = new JMenuItem("Copy instance");
      JMenuItem menuEditionCutInstance = new JMenuItem("Cut instance");
      JMenuItem menuEditionPasteInstance = new JMenuItem("Paste instance");
      JMenuItem menuEditionChangeEquip = new JMenuItem("Equip location");
      JMenuItem menuEditionInformations  = new JMenuItem("Informations");

  public ObjectTreeDialog(Frame owner, String title, boolean modal, ObjectTree objectTree)
  {
     super(owner, title, modal);
     this.objectTree = objectTree;

     scrollPane = new JScrollPane(objectTree.treeOfObjects, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
     getContentPane().setLayout(new BorderLayout());
     getContentPane().add(menu, BorderLayout.NORTH);
       menu.add(menuAdd);
         menuAdd.add(menuNewMobile);
           menuNewMobile.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_M, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
         menuAdd.add(menuNewObject);
           menuNewObject.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
         menuAdd.add(menuAddMobileInstance);
         menuAdd.add(menuAddObjectInstance);
         menuAdd.add(menuAddExternalMobileInstance);
         menuAdd.add(menuAddExternalObjectInstance);
       menu.add(menuRemove);
         menuRemove.add(menuRemoveMobile);
         menuRemove.add(menuRemoveObject);
         menuRemove.add(menuRemoveInstance);
           menuRemoveInstance.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0));
      menu.add(menuEdition);
         menuEdition.add(menuEditionUndo);
           menuEditionUndo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
         menuEdition.add(menuEditionRedo);
           menuEditionRedo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask() | InputEvent.SHIFT_MASK));
         menuEdition.addSeparator();
         menuEdition.add(menuEditionCopyInstance);
           menuEditionCopyInstance.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
         menuEdition.add(menuEditionCutInstance);
           menuEditionCutInstance.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
         menuEdition.add(menuEditionPasteInstance);
           menuEditionPasteInstance.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
         menuEdition.addSeparator();
         menuEdition.add(menuEditionChangeEquip);
           menuEditionChangeEquip.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
         menuEdition.add(menuEditionInformations);
           menuEditionInformations.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));

    getContentPane().add(scrollPane, BorderLayout.CENTER);

    addWindowListener(this);

    menuNewMobile.addActionListener(this);
    menuNewObject.addActionListener(this);
    menuAddMobileInstance.addActionListener(this);
    menuAddObjectInstance.addActionListener(this);
    menuAddExternalMobileInstance.addActionListener(this);
    menuAddExternalObjectInstance.addActionListener(this);
    menuRemoveMobile.addActionListener(this);
    menuRemoveObject.addActionListener(this);
    menuRemoveInstance.addActionListener(this);
    menuEditionUndo.addActionListener(this);
    menuEditionRedo.addActionListener(this);
    menuEditionCopyInstance.addActionListener(this);
    menuEditionCutInstance.addActionListener(this);
    menuEditionPasteInstance.addActionListener(this);
    menuEditionChangeEquip.addActionListener(this);
    menuEditionInformations.addActionListener(this);

    objectTree.treeOfObjects.addTreeSelectionListener(this);
    pack();
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension mySize = this.getSize();
    this.setLocation((int) (screenSize.getWidth()-mySize.getWidth())/2,
                     (int) (screenSize.getHeight()-mySize.getHeight())/2
                    );
  }

  public void valueChanged(TreeSelectionEvent e)
  {
    DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode) e.getPath().getLastPathComponent();
    enableMenus(selectedNode);
  }

  public void clear()
  {
    actionStack.clear(); //free the stack
    objectTree.clear();
    actionStack.clear(); //we redo it because freeing map could have pushed some elements
  }

  public void enableMenus()
  {
     DefaultMutableTreeNode selectedNode = null;
     try
     {
        TreePath treePath = objectTree.treeOfObjects.getSelectionPath();
        selectedNode = (DefaultMutableTreeNode) treePath.getLastPathComponent();
     }
     catch(Exception x) {return;}
     enableMenus(selectedNode);
  }

  public void enableMenus(DefaultMutableTreeNode selectedNode)
  {
    menuEditionUndo.setEnabled(!actionStack.undoStack.isEmpty());
    menuEditionRedo.setEnabled(!actionStack.redoStack.isEmpty());

    AnatoliaRoom room = AreaBuilderConsts.getRoomNode(selectedNode);
    InstanceOfMobile mobile = AreaBuilderConsts.getMobileNode(selectedNode);
    InstanceOfObject object = AreaBuilderConsts.getObjectNode(selectedNode);
    menuNewMobile.setEnabled((selectedNode != null));
    menuNewObject.setEnabled((selectedNode != null));
    menuAddMobileInstance.setEnabled((selectedNode != null));
    menuAddObjectInstance.setEnabled((selectedNode != null));
    menuAddExternalMobileInstance.setEnabled((selectedNode != null));
    menuAddExternalObjectInstance.setEnabled((selectedNode != null));
    menuRemoveMobile.setEnabled(true);
    menuRemoveObject.setEnabled(true);
    menuRemoveInstance.setEnabled((selectedNode != null) && ((mobile != null) || (object != null)));
    menuEditionCopyInstance.setEnabled((selectedNode != null) && ((mobile != null) || (object != null)));
    menuEditionCutInstance.setEnabled((selectedNode != null) && ((mobile != null) || (object != null)));
    menuEditionPasteInstance.setEnabled((selectedNode != null));
    menuEditionChangeEquip.setEnabled((selectedNode != null) && (object != null));
    menuEditionInformations.setEnabled((selectedNode != null));
  }

  public void windowClosing(WindowEvent e)
  {
     objectTree.areaMap.mainScreen.menuDisplayTree.setSelected(false);
  }

  public void windowClosed(WindowEvent e) {}
  public void windowOpened(WindowEvent e) {enableMenus();}
  public void windowDeactivated(WindowEvent e) {}
  public void windowActivated(WindowEvent e) {enableMenus();}
  public void windowDeiconified(WindowEvent e) {}
  public void windowIconified(WindowEvent e) {}

  public void actionPerformed(ActionEvent e)
  {
     if (e.getSource() == menuNewMobile)
     {
        newMobile();
     }
     else if (e.getSource() == menuNewObject)
     {
        newObject();
     }
     else if (e.getSource() == menuAddMobileInstance)
     {
        addMobileInstance();
     }
     else if (e.getSource() == menuAddExternalMobileInstance)
     {
        addExternalMobileInstance();
     }
     else if (e.getSource() == menuAddExternalObjectInstance)
     {
        addExternalObjectInstance();
     }
     else if (e.getSource() == menuRemoveMobile)
     {
        removeMobile();
     }
     else if (e.getSource() == menuRemoveObject)
     {
        removeObject();
     }
     else if (e.getSource() == menuRemoveInstance)
     {
        removeInstance();
     }
     else if (e.getSource() == menuEditionUndo)
     {
        actionStack.undoAction();
        setUndoState(!actionStack.undoStack.isEmpty());
        setRedoState(!actionStack.redoStack.isEmpty());
        objectTree.areaMap.mainScreen.setMustSave(true);
     }
     else if (e.getSource() == menuEditionRedo)
     {
        actionStack.redoAction();
        setUndoState(!actionStack.undoStack.isEmpty());
        setRedoState(!actionStack.redoStack.isEmpty());
        objectTree.areaMap.mainScreen.setMustSave(true);
     }
     else if (e.getSource() == menuEditionCopyInstance)
     {
        copyInstance();
     }
     else if (e.getSource() == menuEditionCutInstance)
     {
        cutInstance();
     }
     else if (e.getSource() == menuEditionPasteInstance)
     {
        pasteInstance();
     }
     else if (e.getSource() == menuEditionChangeEquip)
     {
        changeEquipLocation();
     }
     else if (e.getSource() == menuEditionInformations)
     {
        DefaultMutableTreeNode current = null;
        try
        {
           TreePath treePath = objectTree.treeOfObjects.getSelectionPath();
           current = (DefaultMutableTreeNode) treePath.getLastPathComponent();
        }
        catch(Exception x) {return;}
        try
        {
           objectTree.areaMap.anatoliaRoomDialog.showFor(((AnatoliaRoom) current).myRoom);
        }
        catch(Exception x) {}
        try
        {
           objectTree.areaMap.anatoliaMobileDialog.showFor(((InstanceOfMobile) current).anatoliaMobile);
        }
        catch(Exception x) {}
        try
        {
           InstanceOfObject object = (InstanceOfObject) current;
           objectTree.areaMap.anatoliaObjectDialog.showFor(object.anatoliaObject);
           if (object.anatoliaObject.type != AnatoliaConsts.OBJECT_TYPE_CONTAINER)
           {
             for(int i=0 ; i<object.getChildCount() ; ++i)
             {
               removeObjectInstanceWithoutAsking((InstanceOfObject) object.getChildAt(i));
             }
           }
        }
        catch(Exception x) {}
     }
  }

  public void setUndoState(boolean undoState) {menuEditionUndo.setEnabled(undoState);}
  public void setRedoState(boolean redoState) {menuEditionRedo.setEnabled(redoState);}

  private void copyInstance()
  {
     AbstractInstance current = null;
     try
     {
        TreePath treePath = objectTree.treeOfObjects.getSelectionPath();
        current = (AbstractInstance) treePath.getLastPathComponent();
        if (current == null) return;
     }
     catch(Exception e) {return;}
     AbstractInstance copy = (AbstractInstance) current.clone();
     bufferTreeModel = new DefaultTreeModel(copy);
     bufferTree = new JTree(bufferTreeModel);
     copyChildrenOfToRoot(current, copy);
  }

  private void copyChildrenOfToRoot(AbstractInstance parent, DefaultMutableTreeNode root)
  {
    AbstractInstance child = null;
    for(int i=0 ; i<parent.getChildCount() ; ++i)
    {
      child = (AbstractInstance) parent.getChildAt(i);
      AbstractInstance copy = (AbstractInstance) child.clone();
      bufferTreeModel.insertNodeInto(copy, root, root.getChildCount());
      copyChildrenOfToRoot(child, copy);
    }
  }

  private void pasteChildrenOfToRoot(AbstractInstance parent, DefaultMutableTreeNode root)
  {
    InstanceOfObject child = null;
    for(int i=0 ; i<parent.getChildCount() ; ++i)
    {
       child = (InstanceOfObject) parent.getChildAt(i);
       InstanceOfObject copy = (InstanceOfObject) child.clone();
       actionStack.doAction(new UndoableActionObjectTreeNodeAdd(objectTree, copy, root, UndoableAction.AUTOMATIC_ACTION));
       actionStack.doAction(new UndoableActionObjectTreeInstanceOfObjectAdd(objectTree, copy, UndoableAction.AUTOMATIC_ACTION));
       pasteChildrenOfToRoot(child, copy);
    }
  }

  private void pasteInstance()
  {
     DefaultMutableTreeNode current = null;
     try
     {
        TreePath treePath = objectTree.treeOfObjects.getSelectionPath();
        current = (DefaultMutableTreeNode) treePath.getLastPathComponent();
        if (current == null) return;
     }
     catch(Exception e) {return;}

     //paste a mobile
     try
     {
       InstanceOfMobile mobile = (InstanceOfMobile) bufferTreeModel.getRoot();
       if (AreaBuilderConsts.getRoomNode(current) == null)
       {
         current = (DefaultMutableTreeNode) current.getParent();
         if (AreaBuilderConsts.getRoomNode(current) == null) return;
       }
       //<current> is now the room
       InstanceOfMobile copy = (InstanceOfMobile) mobile.clone();
       actionStack.doAction(new UndoableActionObjectTreeNodeAdd(objectTree, copy, current));
       actionStack.doAction(
          new UndoableActionObjectTreeInstanceOfMobileAdd(
             objectTree,
             copy,
             AreaBuilderConsts.getRoomNode(current),
             UndoableAction.AUTOMATIC_ACTION
          )
       );
       pasteChildrenOfToRoot(mobile, copy);
       objectTree.areaMap.mainScreen.setMustSave(true);
       setUndoState(true);
     }
     catch(Exception e) {}

     //paste an object
     try
     {
       InstanceOfObject object = (InstanceOfObject) bufferTreeModel.getRoot();
       if (AreaBuilderConsts.getObjectNode(current) != null)
       {
         if (AreaBuilderConsts.getObjectNode(current).anatoliaObject.type != AnatoliaConsts.OBJECT_TYPE_CONTAINER)
           current = (DefaultMutableTreeNode) current.getParent();
       }
       //<current> is now the parent
       InstanceOfObject copy = (InstanceOfObject) object.clone();
       actionStack.doAction(new UndoableActionObjectTreeNodeAdd(objectTree, copy, current));
       actionStack.doAction(
          new UndoableActionObjectTreeInstanceOfObjectAdd(
             objectTree,
             copy,
             UndoableAction.AUTOMATIC_ACTION
          )
       );
       pasteChildrenOfToRoot(object, copy);
       objectTree.areaMap.mainScreen.setMustSave(true);
       setUndoState(true);
     }
     catch(Exception e) {}
  }

  private void cutInstance()
  {
     copyInstance();
     removeInstance();
  }

  private void newMobile()
  {
     DefaultMutableTreeNode current = null;
     try
     {
        TreePath treePath = objectTree.treeOfObjects.getSelectionPath();
        current = (DefaultMutableTreeNode) treePath.getLastPathComponent();
     }
     catch(Exception e) {return;}

     if (AreaBuilderConsts.getRoomNode(current) == null)
     {
        current = (DefaultMutableTreeNode) current.getParent();
        if (AreaBuilderConsts.getRoomNode(current) == null) return;
     }
     //<current> is now the room

     AnatoliaMobile anatoliaMobile = new AnatoliaMobile(objectTree.areaMap.theMobiles);
     objectTree.areaMap.anatoliaMobileDialog.showFor(anatoliaMobile);
     if (objectTree.areaMap.anatoliaMobileDialog.getLastActionOK())
     {
        actionStack.doAction(new UndoableActionObjectTreeMobileAdd(objectTree, anatoliaMobile));
        objectTree.areaMap.mainScreen.setMustSave(true);
        setUndoState(true);

        InstanceOfMobile newInstance = new InstanceOfMobile(anatoliaMobile);
        actionStack.doAction(
          new UndoableActionObjectTreeInstanceOfMobileAdd(
            objectTree, newInstance, (AnatoliaRoom) current,
            UndoableAction.AUTOMATIC_ACTION
          )
        );
        actionStack.doAction(new UndoableActionObjectTreeNodeAdd(objectTree, newInstance, current, UndoableAction.AUTOMATIC_ACTION));
     }
  }

  private void newObject()
  {
     DefaultMutableTreeNode current = null;
     int equipLocation = 0;
     try
     {
        TreePath treePath = objectTree.treeOfObjects.getSelectionPath();
        current = (DefaultMutableTreeNode) treePath.getLastPathComponent();
     }
     catch(Exception e) {return;}

     //if we are on an object, let's take its parent if it's not a  container
     if (AreaBuilderConsts.getObjectNode(current) != null)
     {
       if (AreaBuilderConsts.getObjectNode(current).anatoliaObject.type != AnatoliaConsts.OBJECT_TYPE_CONTAINER)
         current = (DefaultMutableTreeNode) current.getParent();
     }
     //<current> is now the parent

     AnatoliaObject anatoliaObject = new AnatoliaObject(objectTree.areaMap.theObjects);
     objectTree.areaMap.anatoliaObjectDialog.showFor(anatoliaObject);
     if (objectTree.areaMap.anatoliaObjectDialog.getLastActionOK())
     {
        actionStack.doAction(new UndoableActionObjectTreeObjectAdd(objectTree, anatoliaObject));
        objectTree.areaMap.mainScreen.setMustSave(true);
        setUndoState(true);

         boolean equipAllMobileInstances = false;
         //if parent is a mobile, propose to equip all instances
         if (AreaBuilderConsts.getMobileNode(current) != null)
         {
            EquipingDialog eq = new EquipingDialog();
            if (eq.lastOK == false) return;
            equipLocation = eq.locationBox.getSelectedIndex();
            equipAllMobileInstances = eq.allInstancesCheckBox.isSelected();
         }
         if (equipAllMobileInstances)
         {
           Iterator it = AreaBuilderConsts.getMobileNode(current).anatoliaMobile.instances.iterator();
           InstanceOfMobile currentMob = null;
           while (it.hasNext())
           {
             currentMob = (InstanceOfMobile) it.next();
             InstanceOfObject newInstance = new InstanceOfObject(anatoliaObject, equipLocation);
             actionStack.doAction(
               new UndoableActionObjectTreeInstanceOfObjectAdd(
                 objectTree,
                 newInstance,
                 UndoableAction.AUTOMATIC_ACTION
               )
             );

             actionStack.doAction(new UndoableActionObjectTreeNodeAdd(objectTree, newInstance, currentMob, UndoableAction.AUTOMATIC_ACTION));
           }
         }
         else
         {
           InstanceOfObject newInstance = new InstanceOfObject(anatoliaObject, equipLocation);
           actionStack.doAction(
             new UndoableActionObjectTreeInstanceOfObjectAdd(
               objectTree,
               newInstance,
               UndoableAction.AUTOMATIC_ACTION
             )
           );

           actionStack.doAction(new UndoableActionObjectTreeNodeAdd(objectTree, newInstance, current, UndoableAction.AUTOMATIC_ACTION));
         }
     }
  }

  private void addMobileInstance()
  {
     DefaultMutableTreeNode current = null;
     try
     {
        TreePath treePath = objectTree.treeOfObjects.getSelectionPath();
        current = (DefaultMutableTreeNode) treePath.getLastPathComponent();
     }
     catch(Exception e) {return;}

     if (AreaBuilderConsts.getRoomNode(current) == null)
     {
        current = (DefaultMutableTreeNode) current.getParent();
        if (AreaBuilderConsts.getRoomNode(current) == null) return;
     }
     //<current> is now the room

     //check if exist some mobiles
     Object[] mobiles = objectTree.areaMap.theMobiles.toArray();
     if (mobiles.length == 0) return;

     //ask which mobile to instanciate
     AnatoliaMobile selected = (AnatoliaMobile) JOptionPane.showInputDialog(this, "Existing mobiles",
                                                          "Select an existing mobile",
                                                          JOptionPane.QUESTION_MESSAGE, null,
                                                          mobiles, mobiles[0]);
     if (selected != null)
     {
        InstanceOfMobile newInstance = new InstanceOfMobile(selected);
        actionStack.doAction(
          new UndoableActionObjectTreeInstanceOfMobileAdd(objectTree, newInstance, (AnatoliaRoom) current)
        );
        objectTree.areaMap.mainScreen.setMustSave(true);
        setUndoState(true);

        actionStack.doAction(new UndoableActionObjectTreeNodeAdd(objectTree, newInstance, current, UndoableAction.AUTOMATIC_ACTION));
     }
  }

  private void addExternalMobileInstance()
  {
     DefaultMutableTreeNode current = null;
     try
     {
        TreePath treePath = objectTree.treeOfObjects.getSelectionPath();
        current = (DefaultMutableTreeNode) treePath.getLastPathComponent();
     }
     catch(Exception e) {return;}

     if (AreaBuilderConsts.getRoomNode(current) == null)
     {
        current = (DefaultMutableTreeNode) current.getParent();
        if (AreaBuilderConsts.getRoomNode(current) == null) return;
     }
     //<current> is now the room

     ExternDialog externDialog = new ExternMobsDialog(this, "Add external mobile", true);
     int choice = externDialog.open();
     if (choice == JOptionPane.OK_OPTION)
     {
       int vnum = externDialog.getVnum();
       if (vnum < 0)
       {
         JOptionPane.showMessageDialog(this, "Incorrect vnum", "Incorrect Vnum",JOptionPane.ERROR_MESSAGE);
       }
       else
       {
         String wellKnownName = externDialog.externName;
         AnatoliaMobile mobile = new AnatoliaMobile(objectTree.areaMap.theMobiles);
         mobile.imposeVnum(vnum);
         mobile.setExtern(true);
         actionStack.doAction(
           new UndoableActionObjectTreeMobileAdd(objectTree, mobile)
         );
         InstanceOfMobile newInstance = new InstanceOfMobile(mobile);
         if (!wellKnownName.trim().equals("")) mobile.nameList = wellKnownName;
         actionStack.doAction(
           new UndoableActionObjectTreeInstanceOfMobileAdd(objectTree, newInstance, (AnatoliaRoom) current, UndoableAction.AUTOMATIC_ACTION)
         );
         objectTree.areaMap.mainScreen.setMustSave(true);
         setUndoState(true);

         actionStack.doAction(new UndoableActionObjectTreeNodeAdd(objectTree, newInstance, current, UndoableAction.AUTOMATIC_ACTION));
       }
     }
  }

  private void addExternalObjectInstance()
  {
     DefaultMutableTreeNode current = null;
     int equipLocation = 0;
     try
     {
        TreePath treePath = objectTree.treeOfObjects.getSelectionPath();
        current = (DefaultMutableTreeNode) treePath.getLastPathComponent();
     }
     catch(Exception e) {return;}

     //if we are on an object, let's take its parent if it's not a  container
     if (AreaBuilderConsts.getObjectNode(current) != null)
     {
       if (AreaBuilderConsts.getObjectNode(current).anatoliaObject.type != AnatoliaConsts.OBJECT_TYPE_CONTAINER)
         current = (DefaultMutableTreeNode) current.getParent();
     }
     //<current> is now the parent

     ExternDialog externDialog = new ExternObjsDialog(this, "Add external object", true);
     int choice = externDialog.open();
     if (choice == JOptionPane.OK_OPTION)
     {
       int vnum = externDialog.getVnum();
       if (vnum < 0)
       {
         JOptionPane.showMessageDialog(this, "Incorrect vnum", "Incorrect Vnum",JOptionPane.ERROR_MESSAGE);
       }
       else
       {
         String wellKnownName = externDialog.externName;
         AnatoliaObject object = new AnatoliaObject(objectTree.areaMap.theObjects);
         object.imposeVnum(vnum);
         object.setExtern(true);
         actionStack.doAction(
           new UndoableActionObjectTreeObjectAdd(objectTree, object)
         );
         InstanceOfObject newInstance = new InstanceOfObject(object,equipLocation);
         if (!wellKnownName.trim().equals("")) object.name = wellKnownName;
         actionStack.doAction(
           new UndoableActionObjectTreeInstanceOfObjectAdd(objectTree, newInstance, UndoableAction.AUTOMATIC_ACTION)
         );
         objectTree.areaMap.mainScreen.setMustSave(true);
         setUndoState(true);

         actionStack.doAction(new UndoableActionObjectTreeNodeAdd(objectTree, newInstance, current, UndoableAction.AUTOMATIC_ACTION));
       }
     }
  }

  private void addObjectInstance()
  {
     DefaultMutableTreeNode current = null;
     int equipLocation = 0;
     try
     {
        TreePath treePath = objectTree.treeOfObjects.getSelectionPath();
        current = (DefaultMutableTreeNode) treePath.getLastPathComponent();
     }
     catch(Exception e) {return;}

     //if we are on an object, let's take its parent if it's not a  container
     if (AreaBuilderConsts.getObjectNode(current) != null)
     {
       if (AreaBuilderConsts.getObjectNode(current).anatoliaObject.type != AnatoliaConsts.OBJECT_TYPE_CONTAINER)
         current = (DefaultMutableTreeNode) current.getParent();
     }
     //<current> is now the parent

     //check if exists some objetcs
     Object[] objects = objectTree.areaMap.theObjects.toArray();
     if (objects.length == 0) return;

     //ask which one to instanciate
     AnatoliaObject selected = (AnatoliaObject) JOptionPane.showInputDialog(this, "Existing objects",
                                                          "Select an existing object",
                                                          JOptionPane.QUESTION_MESSAGE, null,
                                                          objects, objects[0]);
     if (selected != null)
     {
        //if parent is a mobile, propose to equip all instances
        boolean equipAllMobileInstances = false;
        if (AreaBuilderConsts.getMobileNode(current) != null)
        {
          EquipingDialog eq = new EquipingDialog();
          if (eq.lastOK == false) return;
          equipLocation = eq.locationBox.getSelectedIndex();
          equipAllMobileInstances = eq.allInstancesCheckBox.isSelected();
        }

        //////////IMPORTANT////////
        //this allows to put automatic actions after
        actionStack.doAction(new UndoableActionObjectTree(objectTree));
        objectTree.areaMap.mainScreen.setMustSave(true);
        setUndoState(true);
        //DO NOT REMOVE
        ///////////////////////////

        if (equipAllMobileInstances)
        {
           Iterator it = AreaBuilderConsts.getMobileNode(current).anatoliaMobile.instances.iterator();
           InstanceOfMobile currentMob = null;
           while (it.hasNext())
           {
             currentMob = (InstanceOfMobile) it.next();
             InstanceOfObject newInstance = new InstanceOfObject(selected,equipLocation);
             actionStack.doAction(
               new UndoableActionObjectTreeInstanceOfObjectAdd(
                  objectTree, newInstance,UndoableAction.AUTOMATIC_ACTION)
             );
             actionStack.doAction(new UndoableActionObjectTreeNodeAdd(
               objectTree,newInstance, currentMob, UndoableAction.AUTOMATIC_ACTION)
             );
           }
        }
        else
        {
          InstanceOfObject newInstance = new InstanceOfObject(selected,equipLocation);
          actionStack.doAction(
            new UndoableActionObjectTreeInstanceOfObjectAdd(
               objectTree, newInstance,UndoableAction.AUTOMATIC_ACTION)
          );

          actionStack.doAction(new UndoableActionObjectTreeNodeAdd(
            objectTree,newInstance, current, UndoableAction.AUTOMATIC_ACTION)
          );
        }
     }
  }

  private void removeMobile()
  {
     Object[] mobiles = objectTree.areaMap.theMobiles.toArray();
     if (mobiles.length == 0) return;

     AnatoliaMobile selected = (AnatoliaMobile) JOptionPane.showInputDialog(this, "Existing mobiles",
                                                          "Select an existing mobile",
                                                          JOptionPane.QUESTION_MESSAGE, null,
                                                          mobiles, mobiles[0]);
     if (selected != null)
     {
        int choice = JOptionPane.showConfirmDialog(this,"All instances of this mobile"+"\n"+
                                                        "will be lost."+"\n"+
                                                        "Do you want to continue?",
                                      "Mobile Deletion", JOptionPane.YES_NO_OPTION,
                                      JOptionPane.WARNING_MESSAGE);

        if (choice == JOptionPane.NO_OPTION) return;

        actionStack.doAction(new UndoableActionObjectTreeMobileRemove(objectTree,selected));
        setUndoState(true);
        Iterator it = selected.instances.iterator();
        while(it.hasNext())
          actionStack.doAction(new UndoableActionObjectTreeNodeRemove(objectTree, (InstanceOfMobile) it.next(), UndoableAction.AUTOMATIC_ACTION));
     }
  }

  private void removeObject()
  {
     Object[] objects = objectTree.areaMap.theObjects.toArray();
     if (objects.length == 0) return;

     AnatoliaObject selected = (AnatoliaObject) JOptionPane.showInputDialog(this, "Existing objects",
                                                          "Select an existing object",
                                                          JOptionPane.QUESTION_MESSAGE, null,
                                                          objects, objects[0]);
     if (selected != null)
     {
        int choice = JOptionPane.showConfirmDialog(this,"All instances of this object"+"\n"+
                                                        "will be lost."+"\n"+
                                                        "Do you want to continue?",
                                      "Object Deletion", JOptionPane.YES_NO_OPTION,
                                      JOptionPane.WARNING_MESSAGE);

        if (choice == JOptionPane.NO_OPTION) return;

        actionStack.doAction(new UndoableActionObjectTreeObjectRemove(objectTree,selected));
        setUndoState(true);
        Iterator it = selected.instances.iterator();
        while(it.hasNext())
          actionStack.doAction(new UndoableActionObjectTreeNodeRemove(objectTree, (InstanceOfObject) it.next(), UndoableAction.AUTOMATIC_ACTION));
     }
  }

  private void removeInstance()
  {
     DefaultMutableTreeNode current = null;
     try
     {
        TreePath treePath = objectTree.treeOfObjects.getSelectionPath();
        current = (DefaultMutableTreeNode) treePath.getLastPathComponent();
     }
     catch(Exception e) {return;}

     try
     {
        InstanceOfMobile instance = (InstanceOfMobile) current;
        removeMobileInstance(instance);
        return;
     }
     catch(Exception e) {}
     try
     {
        InstanceOfObject instance = (InstanceOfObject) current;
        removeObjectInstance(instance);
        return;
     }
     catch(Exception e) {}

  }

  private void removeMobileInstance(InstanceOfMobile instance)
  {
     //////////IMPORTANT////////
     //this allows to put automatic actions after
     actionStack.doAction(new UndoableActionObjectTree(objectTree));
     objectTree.areaMap.mainScreen.setMustSave(true);
     setUndoState(true);
     //DO NOT REMOVE
     ///////////////////////////

     //if mobiles is equiped, deletes equipment
     for(int i=0 ; i<instance.getChildCount() ; ++i)
     {
       removeObjectInstanceWithoutAsking((InstanceOfObject) instance.getChildAt(i));
     }
     actionStack.doAction(
       new UndoableActionObjectTreeNodeRemove(
         objectTree, instance, UndoableAction.AUTOMATIC_ACTION
       )
     );

     actionStack.doAction(
       new UndoableActionObjectTreeInstanceOfMobileRemove(
         objectTree, instance, UndoableAction.AUTOMATIC_ACTION
       )
     );


     //maybe there are no more instance of this mobile
     if (instance.anatoliaMobile.instances.size() == 0)
     {
       int choice = JOptionPane.showConfirmDialog(
                          this, "There are no more instances of Mobile"+"\n"+
                          "<"+instance.anatoliaMobile.nameList+">"+"\n"+
                          "do you want to delete its description ?",
                          "No more instances",
                          JOptionPane.YES_NO_OPTION,
                          JOptionPane.QUESTION_MESSAGE);
       if (choice == JOptionPane.NO_OPTION) return;

       choice = JOptionPane.showConfirmDialog(
                          this, "The description of the mobile"+"\n"+
                          "<"+instance.anatoliaMobile.nameList+">"+"\n"+
                          "will be lost.\n"+
                          "Do you want to continue?",
                          "Mobile Deletion",
                          JOptionPane.YES_NO_OPTION,
                          JOptionPane.WARNING_MESSAGE);
       if (choice == JOptionPane.NO_OPTION) return;
       actionStack.doAction(
         new UndoableActionObjectTreeMobileRemove(
           objectTree, instance.anatoliaMobile, UndoableAction.AUTOMATIC_ACTION
         )
       );
     }
  }
  private void removeObjectInstanceWithoutAsking(DefaultMutableTreeNode node)
  {
     for(int i=0 ; i<node.getChildCount() ; ++i)
     {
       removeObjectInstanceWithoutAsking((DefaultMutableTreeNode) node.getChildAt(i));
     }

     actionStack.doAction(
       new UndoableActionObjectTreeNodeRemove(
         objectTree, node, UndoableAction.AUTOMATIC_ACTION
       )
     );
     try
     {
       InstanceOfObject instance = (InstanceOfObject) node;
       actionStack.doAction(
         new UndoableActionObjectTreeInstanceOfObjectRemove(
           objectTree, instance, UndoableAction.AUTOMATIC_ACTION
         )
       );
     }
     catch(Exception e) {}
  }

  private void removeObjectInstance(InstanceOfObject instance)
  {
     //////////IMPORTANT////////
     //this allows to put automatic actions after
     actionStack.doAction(new UndoableActionObjectTree(objectTree));
     objectTree.areaMap.mainScreen.setMustSave(true);
     setUndoState(true);
     //DO NOT REMOVE
     ///////////////////////////

     try //if this object equips a mobile
     {
       InstanceOfMobile mobile = (InstanceOfMobile) instance.getTheParent();
       LinkedList similarInstances = new LinkedList();
       Iterator it = instance.anatoliaObject.instances.iterator();
       InstanceOfObject keepIt = null;
       InstanceOfMobile keepItMobile = null;
       //looks for other instance of the objects equiping instances of the same mobile
       while(it.hasNext())
       {
         keepIt = (InstanceOfObject) it.next();
         keepItMobile = AreaBuilderConsts.getMobileNode(keepIt.getTheParent());
         if ((keepItMobile != null) &&
             (keepItMobile.anatoliaMobile == mobile.anatoliaMobile)
            )
         {
           similarInstances.add(keepIt);
         }
       }
       //similar instances CONTAINS the "generating" instance

       if (similarInstances.size() > 1)
       {
          int choice = JOptionPane.showConfirmDialog(this,
                     "Several instances of this object equip the instances of the mobile\n"+
                     "Do you want to delete the other similar instances?\n",
                     "Deletion",
                     JOptionPane.YES_NO_CANCEL_OPTION,
                     JOptionPane.QUESTION_MESSAGE);
          if (choice == JOptionPane.CANCEL_OPTION) return;
          if (choice == JOptionPane.NO_OPTION)
          {
            removeObjectInstanceWithoutAsking(instance);
          }
          else if (choice == JOptionPane.YES_OPTION)
          {
              it = similarInstances.iterator();
              while(it.hasNext())
              {
                removeObjectInstanceWithoutAsking((InstanceOfObject) it.next());
              }
          }
       }
       else removeObjectInstanceWithoutAsking(instance);
     }
     catch(Exception e)
     {
        removeObjectInstanceWithoutAsking(instance);
     }

     //maybe there are no more instance of this object
     if (instance.anatoliaObject.instances.size() == 0)
     {
       int choice = JOptionPane.showConfirmDialog(
                          this, "There are no more instances of Object"+"\n"+
                          "<"+instance.anatoliaObject.name+"\n"+
                          "do you want to delete its description ?",
                          "No more instances",
                          JOptionPane.YES_NO_OPTION,
                          JOptionPane.QUESTION_MESSAGE);
       if (choice == JOptionPane.NO_OPTION) return;

       choice = JOptionPane.showConfirmDialog(
                          this, "The description of object"+"\n"+
                          "<"+instance.anatoliaObject.name+">"+"\n"+
                          "will be lost."+"\n"+
                          "Do you want to continue?",
                          "Object Deletion",
                          JOptionPane.YES_NO_OPTION,
                          JOptionPane.WARNING_MESSAGE);
       if (choice == JOptionPane.NO_OPTION) return;

       actionStack.doAction(
         new UndoableActionObjectTreeObjectRemove(
            objectTree, instance.anatoliaObject, UndoableAction.AUTOMATIC_ACTION
         )
       );
     }
  }

  private void changeEquipLocation()
  {
     try
     {
        DefaultMutableTreeNode current = null;
        InstanceOfMobile mobile = null;
        TreePath treePath = objectTree.treeOfObjects.getSelectionPath();
        current = (DefaultMutableTreeNode) treePath.getLastPathComponent();
        InstanceOfObject instance = (InstanceOfObject) current;
        mobile = (InstanceOfMobile) current.getParent();

        String[] wearTexts = AnatoliaConsts.getWearTexts();
        String wear = (String) JOptionPane.showInputDialog(this, "Select a location","Equip a mobile",
                                                           JOptionPane.QUESTION_MESSAGE,null,
                                                           wearTexts, wearTexts[instance.equipLocation]);
        if (wear != null) instance.equipLocation = AreaBuilderConsts.findInTab(wearTexts, wear);
     }
     catch(Exception e) {return;}
  }
}