/*********************************************************************
** AreaBuilder is a program designed to build maps for MUDS like    **
** Anatolia.                                                        **
** Copyright (C) 2003 Pierre Chatelier                              **
**                                                                  **
** This program is free software; you can redistribute              **
** it and/or modify it under the terms of the GNU                   **
** General Public License as published by the                       **
** Free Software Foundation, version 2.                             **
**                                                                  **
** This program is distributed in the hope that it will be          **
** useful, but WITHOUT ANY WARRANTY; without even the implied       **
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. **
** See the GNU General Public License for more details.             **
**                                                                  **
** You should have received a copy of the                           **
** GNU General Public License along with this program; if not,      **
** write to the Free Software Foundation,                           **
** Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA      **
*********************************************************************/
package areabuilder;

import java.awt.*;
import java.util.Stack;

public class UndoableActionStack
{
  Stack undoStack;
  Stack redoStack;

  public UndoableActionStack()
  {
    undoStack = new Stack();
    redoStack = new Stack();
  }

  public void clear()
  {
     undoStack.clear();
     redoStack.clear();
  }

  private void undoAutomaticActions()
  {
    UndoableAction action = null;
    boolean checkNextAction = true;
    do
    {
       checkNextAction = !undoStack.empty();
       if (checkNextAction)
       {
          action = (UndoableAction) undoStack.peek();
          checkNextAction = action.isAutomatic();
          if (checkNextAction)
          {
             action = (UndoableAction) undoStack.pop();
             action.undoAction();
             redoStack.push(action);
          }
       }
    } while (checkNextAction);
  }

  private void redoAutomaticActions()
  {
    UndoableAction action = null;
    boolean checkNextAction = true;
    do
    {
       checkNextAction = !redoStack.empty();
       if (checkNextAction)
       {
          action = (UndoableAction) redoStack.peek();
          checkNextAction = action.isAutomatic();
          if (checkNextAction)
          {
             action = (UndoableAction) redoStack.pop();
             action.redoAction();
             undoStack.push(action);
          }
       }
    } while (checkNextAction);
  }

  public void doAction(UndoableAction action)
     { doAction(action, UndoableAction.ALLOW_UNDO_YES);}

  public void doAction(UndoableAction action, boolean allowUndo)
  {
    redoStack.clear();
    if (allowUndo) undoStack.push(action);
    action.doAction();
  }

  public void undoAction()
  {
    undoAutomaticActions();

    if (undoStack.empty() == false)
    {
       UndoableAction action = (UndoableAction) undoStack.pop();
       action.undoAction();
       redoStack.push(action);
    }
  }

  public void undoAllAction()
  {
    UndoableAction action = null;
    while(undoStack.empty() == false)
    {
      action = (UndoableAction) undoStack.pop();
      action.undoAction();
      redoStack.push(action);
    }
  }

  public void redoAction()
  {
    UndoableAction action = null;
    if (redoStack.empty() == false)
    {
       action = (UndoableAction) redoStack.pop();
       action.redoAction();
       undoStack.push(action);
    }
    redoAutomaticActions();
  }

  public void redoAllAction()
  {
    UndoableAction action = null;
    while(redoStack.empty() == false)
    {
      action = (UndoableAction) redoStack.pop();
      action.redoAction();
      undoStack.push(action);
    }
  }
}